//-----------------------------------------------------------------------
// Copyright (c) 2020 Snap Inc.
//-----------------------------------------------------------------------

#include <required.glsl>
#include <std_texture.glsl>

uniform sampler2D baseTex;
uniform vec4      baseTexSize;

uniform float fMipBias;
uniform float mipLevel;
uniform float sampleCount;

varying vec2 varTex1;

#include "includes/studioUtils.glsl"
#include "includes/envProviderUtils.glsl"

#ifdef VERTEX_SHADER

attribute vec2 texture0;

void main() {    
    varTex1 = texture0;

    gl_Position = vec4((texture0 * 2.0 - 1.0), 0.0, 1.0);
}

#endif // VERTEX_SHADER


#ifdef FRAGMENT_SHADER

vec4 filterEnvmap(vec3 vNormal, float fRoughness) {

    float fTotalWeight = 0.0;
    vec3 vPrefilteredColor = vec3(0.0);
    int iNumSamples = int(sampleCount);

    for (int i = 0; i < iNumSamples; i++)
    {
        vec2 vXi = hammersley(i, iNumSamples);

        vec3 vHalf = normalize(importanceSampleGGX(vXi, fRoughness, vNormal));
        float NoH = saturate(dot(vNormal, vHalf));

        vec3 vLight = normalize(2.0 * NoH * vHalf - vNormal);
        float NoL = saturate(dot(vNormal, vLight));

        if (NoL > 0.0)
        {
            float fPdf = Dggx(NoH, fRoughness) * 0.25;

            // Solid angle represented by this sample
            float fOmegaS = 1.0 / (float(iNumSamples) * fPdf);

            vec2 uv = calcPanoramicTexCoordsFromDir(vLight);
            // float fOmegaP = 0.000096;
            float fOmegaP = abs(0.0003 * sin(PI * uv.y + 0.01227183594));
            
            float fMipLevel = max(0.5 * log2( fOmegaS/ fOmegaP) + fMipBias, 0.0);
            fMipLevel = clamp(fMipLevel, 0.0, 5.0);

            uv = calcSeamlessPanoramicUvsForConvolution(uv, baseTexSize.xy, fMipLevel);
            vPrefilteredColor += decodeRGBD(sc_texture2DLod(baseTex, baseTexSize.xy, uv, fMipLevel)) * NoL;

            fTotalWeight += NoL;
        }
    }

    return encodeRGBD(vPrefilteredColor / fTotalWeight);
}

void main() {
    float MIPToRoughness[6];
    MIPToRoughness[0] = 0.0;
    MIPToRoughness[1] = 0.09;
    MIPToRoughness[2] = 0.25;
    MIPToRoughness[3] = 0.46;
    MIPToRoughness[4] = 0.72;
    MIPToRoughness[5] = 1.0;

    vec3 nrm = normalize(calcDirFromPanoramicTexCoords(stretchCoordToOne(varTex1, baseTexSize.zw, mipLevel)));

    gl_FragColor = filterEnvmap(nrm, MIPToRoughness[int(mipLevel)]);
}

#endif // FRAGMENT_SHADER
