#pragma once
#if defined VERTEX_SHADER
#include <std2.glsl>

uniform mat4 sc_ProjectorMatrix;
uniform float sc_ShadowDensity;
uniform vec4 sc_ShadowColor;
uniform mediump sampler2D sc_ShadowTexture;

varying vec2 varShadowTex;

vec2 getProjectedTexCoords(vec4 vertexPosition)
{
    vec4 worldPos;
    #if (sc_RenderingSpace == 2)
    {
        worldPos = vertexPosition;
    }
    #else // #if (sc_RenderingSpace == 2)
    {
        #if (sc_RenderingSpace == 1)
        {
            worldPos = sc_ModelMatrix * vertexPosition;
        }
        #endif // #if (sc_RenderingSpace == 1)
    }
    #endif // #else // #if (sc_RenderingSpace == 2)
    vec4 projectedUVs = sc_ProjectorMatrix * worldPos;
    return ((projectedUVs.xy / vec2(projectedUVs.w)) * 0.5) + vec2(0.5);
}

#elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
#include <std2.glsl>
precision mediump float;
precision highp int;

uniform highp float sc_ShadowDensity;
uniform highp vec4 sc_ShadowColor;
uniform mediump sampler2D sc_ShadowTexture;
uniform highp mat4 sc_ProjectorMatrix;

varying highp vec2 varShadowTex;

highp float getShadowAlpha(highp vec4 color)
{
    highp float shadowAlpha = 1.0;
    #if (((sc_BlendMode_Normal == 1) || (sc_BlendMode_AlphaToCoverage == 1)) || (sc_BlendMode_PremultipliedAlphaHardware == 1))
    {
        shadowAlpha = color.w;
    }
    #else // #if (((sc_BlendMode_Normal == 1) || (sc_BlendMode_AlphaToCoverage == 1)) || (sc_BlendMode_PremultipliedAlphaHardware == 1))
    {
        #if (sc_BlendMode_PremultipliedAlpha)
        {
            shadowAlpha = clamp(color.w * 2.0, 0.0, 1.0);
        }
        #else // #if (sc_BlendMode_PremultipliedAlpha)
        {
            #if (sc_BlendMode_AddWithAlphaFactor)
            {
                shadowAlpha = clamp(dot(color.xyz, vec3(color.w)), 0.0, 1.0);
            }
            #else // #if (sc_BlendMode_AddWithAlphaFactor)
            {
                #if (sc_BlendMode_AlphaTest)
                {
                    shadowAlpha = 1.0;
                }
                #else // #if (sc_BlendMode_AlphaTest)
                {
                    #if (sc_BlendMode_Multiply)
                    {
                        shadowAlpha = (1.0 - dot(color.xyz, vec3(0.3333300054073333740234375))) * color.w;
                    }
                    #else // #if (sc_BlendMode_Multiply)
                    {
                        #if (sc_BlendMode_MultiplyOriginal)
                        {
                            shadowAlpha = (1.0 - clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0)) * color.w;
                        }
                        #else // #if (sc_BlendMode_MultiplyOriginal)
                        {
                            #if (sc_BlendMode_ColoredGlass)
                            {
                                shadowAlpha = clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0) * color.w;
                            }
                            #else // #if (sc_BlendMode_ColoredGlass)
                            {
                                #if (sc_BlendMode_Add)
                                {
                                    shadowAlpha = clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0);
                                }
                                #else // #if (sc_BlendMode_Add)
                                {
                                    #if (sc_BlendMode_AddWithAlphaFactor)
                                    {
                                        shadowAlpha = clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0) * color.w;
                                    }
                                    #else // #if (sc_BlendMode_AddWithAlphaFactor)
                                    {
                                        #if (sc_BlendMode_Screen)
                                        {
                                            shadowAlpha = dot(color.xyz, vec3(0.3333300054073333740234375)) * color.w;
                                        }
                                        #else // #if (sc_BlendMode_Screen)
                                        {
                                            #if (sc_BlendMode_Min)
                                            {
                                                shadowAlpha = 1.0 - clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0);
                                            }
                                            #else // #if (sc_BlendMode_Min)
                                            {
                                                #if (sc_BlendMode_Max)
                                                {
                                                    shadowAlpha = clamp(dot(color.xyz, vec3(1.0)), 0.0, 1.0);
                                                }
                                                #endif // #if (sc_BlendMode_Max)
                                            }
                                            #endif // #else // #if (sc_BlendMode_Min)
                                        }
                                        #endif // #else // #if (sc_BlendMode_Screen)
                                    }
                                    #endif // #else // #if (sc_BlendMode_AddWithAlphaFactor)
                                }
                                #endif // #else // #if (sc_BlendMode_Add)
                            }
                            #endif // #else // #if (sc_BlendMode_ColoredGlass)
                        }
                        #endif // #else // #if (sc_BlendMode_MultiplyOriginal)
                    }
                    #endif // #else // #if (sc_BlendMode_Multiply)
                }
                #endif // #else // #if (sc_BlendMode_AlphaTest)
            }
            #endif // #else // #if (sc_BlendMode_AddWithAlphaFactor)
        }
        #endif // #else // #if (sc_BlendMode_PremultipliedAlpha)
    }
    #endif // #else // #if (((sc_BlendMode_Normal == 1) || (sc_BlendMode_AlphaToCoverage == 1)) || (sc_BlendMode_PremultipliedAlphaHardware == 1))
    return shadowAlpha;
}

highp vec4 evaluateShadowCasterColor(highp vec4 shadowCasterColor)
{
    highp vec4 param = shadowCasterColor;
    highp float alpha = getShadowAlpha(param);
    highp float shadowTerm = sc_ShadowDensity * alpha;
    highp vec3 shadowColor = mix(sc_ShadowColor.xyz, sc_ShadowColor.xyz * shadowCasterColor.xyz, vec3(sc_ShadowColor.w));
    return vec4(shadowColor.x, shadowColor.y, shadowColor.z, shadowTerm);
}

highp float getShadowTexClipFactor()
{
    highp vec2 uv = abs(varShadowTex - vec2(0.5));
    highp float clipFactor = max(uv.x, uv.y);
    return step(clipFactor, 0.5);
}

highp vec3 evaluateShadow()
{
    highp vec4 shadowSample = texture2D(sc_ShadowTexture, varShadowTex) * getShadowTexClipFactor();
    highp vec3 shadowColor = mix(sc_ShadowColor.xyz, sc_ShadowColor.xyz * shadowSample.xyz, vec3(sc_ShadowColor.w));
    highp float shadowDensity = shadowSample.w * sc_ShadowDensity;
    return mix(vec3(1.0), shadowColor, vec3(shadowDensity));
}

#endif // #elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
