#pragma once
#if defined VERTEX_SHADER
#include <std2_shadows.glsl>
#include <std2.glsl>

struct sc_Vertex_t
{
    vec4 position;
    vec3 normal;
    vec3 tangent;
    vec2 texture0;
    vec2 texture1;
};

#ifndef sc_SkinBonesCount
#define sc_SkinBonesCount 0
#endif
#ifndef sc_VertexBlending
#define sc_VertexBlending 0
#endif
#ifndef sc_VertexBlendingUseNormals
#define sc_VertexBlendingUseNormals 0
#endif
#ifndef sc_DepthBufferMode
#define sc_DepthBufferMode 0
#endif
#ifndef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 0
#endif
uniform vec4 sc_BoneMatrices[300];
uniform mat3 sc_SkinBonesNormalMatrices[100];
uniform vec4 weights0;
uniform vec4 weights1;
uniform float sc_DisableFrustumCullingMarker;
uniform vec4 weights2;

attribute vec4 boneData;
attribute vec3 blendShape0Pos;
attribute vec3 blendShape0Normal;
attribute vec3 blendShape1Pos;
attribute vec3 blendShape1Normal;
attribute vec3 blendShape2Pos;
attribute vec3 blendShape2Normal;
attribute vec3 blendShape3Pos;
attribute vec3 blendShape4Pos;
attribute vec3 blendShape5Pos;
attribute vec4 position;
attribute vec3 normal;
attribute vec4 tangent;
attribute vec2 texture0;
attribute vec2 texture1;
varying vec3 varPos;
varying vec3 varNormal;
varying vec4 varTangent;
varying vec4 varTex01;

vec3 skinVertexPosition(int i, vec4 v)
{
    int j = 3 * i;
    vec4 m0 = sc_BoneMatrices[j + 0];
    vec4 m1 = sc_BoneMatrices[j + 1];
    vec4 m2 = sc_BoneMatrices[j + 2];
    return vec3(dot(m0, v), dot(m1, v), dot(m2, v));
}

void sc_SkinVertex(inout sc_Vertex_t v)
{
    #if (sc_SkinBonesCount > 0)
    {
        vec4 weights = fract(boneData);
        weights.x = 1.0 - dot(weights.yzw, vec3(1.0));
        int index0 = int(boneData.x);
        int index1 = int(boneData.y);
        int index2 = int(boneData.z);
        int index3 = int(boneData.w);
        int param = index0;
        vec4 param_1 = v.position;
        int param_2 = index1;
        vec4 param_3 = v.position;
        int param_4 = index2;
        vec4 param_5 = v.position;
        int param_6 = index3;
        vec4 param_7 = v.position;
        vec3 _271 = (((skinVertexPosition(param, param_1) * weights.x) + (skinVertexPosition(param_2, param_3) * weights.y)) + (skinVertexPosition(param_4, param_5) * weights.z)) + (skinVertexPosition(param_6, param_7) * weights.w);
        v.position = vec4(_271.x, _271.y, _271.z, v.position.w);
        mat3 normalMatrix0 = sc_SkinBonesNormalMatrices[index0];
        mat3 normalMatrix1 = sc_SkinBonesNormalMatrices[index1];
        mat3 normalMatrix2 = sc_SkinBonesNormalMatrices[index2];
        mat3 normalMatrix3 = sc_SkinBonesNormalMatrices[index3];
        v.normal = ((((normalMatrix0 * v.normal) * weights.x) + ((normalMatrix1 * v.normal) * weights.y)) + ((normalMatrix2 * v.normal) * weights.z)) + ((normalMatrix3 * v.normal) * weights.w);
        v.tangent = ((((normalMatrix0 * v.tangent) * weights.x) + ((normalMatrix1 * v.tangent) * weights.y)) + ((normalMatrix2 * v.tangent) * weights.z)) + ((normalMatrix3 * v.tangent) * weights.w);
    }
    #endif // #if (sc_SkinBonesCount > 0)
}

void blendTargetShape(inout sc_Vertex_t v, vec3 position_1, float weight)
{
    vec3 _368 = v.position.xyz + (position_1 * weight);
    v.position = vec4(_368.x, _368.y, _368.z, v.position.w);
}

void blendTargetShapeWithNormal(inout sc_Vertex_t v, vec3 position_1, vec3 normal_1, float weight)
{
    sc_Vertex_t param = v;
    vec3 param_1 = position_1;
    float param_2 = weight;
    blendTargetShape(param, param_1, param_2);
    v = param;
    v.normal += (normal_1 * weight);
}

void sc_BlendVertex(inout sc_Vertex_t v)
{
    #if (sc_VertexBlending)
    {
        #if (sc_VertexBlendingUseNormals)
        {
            sc_Vertex_t param = v;
            vec3 param_1 = blendShape0Pos;
            vec3 param_2 = blendShape0Normal;
            float param_3 = weights0.x;
            blendTargetShapeWithNormal(param, param_1, param_2, param_3);
            v = param;
            sc_Vertex_t param_4 = v;
            vec3 param_5 = blendShape1Pos;
            vec3 param_6 = blendShape1Normal;
            float param_7 = weights0.y;
            blendTargetShapeWithNormal(param_4, param_5, param_6, param_7);
            v = param_4;
            sc_Vertex_t param_8 = v;
            vec3 param_9 = blendShape2Pos;
            vec3 param_10 = blendShape2Normal;
            float param_11 = weights0.z;
            blendTargetShapeWithNormal(param_8, param_9, param_10, param_11);
            v = param_8;
        }
        #else // #if (sc_VertexBlendingUseNormals)
        {
            sc_Vertex_t param_12 = v;
            vec3 param_13 = blendShape0Pos;
            float param_14 = weights0.x;
            blendTargetShape(param_12, param_13, param_14);
            v = param_12;
            sc_Vertex_t param_15 = v;
            vec3 param_16 = blendShape1Pos;
            float param_17 = weights0.y;
            blendTargetShape(param_15, param_16, param_17);
            v = param_15;
            sc_Vertex_t param_18 = v;
            vec3 param_19 = blendShape2Pos;
            float param_20 = weights0.z;
            blendTargetShape(param_18, param_19, param_20);
            v = param_18;
            sc_Vertex_t param_21 = v;
            vec3 param_22 = blendShape3Pos;
            float param_23 = weights0.w;
            blendTargetShape(param_21, param_22, param_23);
            v = param_21;
            sc_Vertex_t param_24 = v;
            vec3 param_25 = blendShape4Pos;
            float param_26 = weights1.x;
            blendTargetShape(param_24, param_25, param_26);
            v = param_24;
            sc_Vertex_t param_27 = v;
            vec3 param_28 = blendShape5Pos;
            float param_29 = weights1.y;
            blendTargetShape(param_27, param_28, param_29);
            v = param_27;
        }
        #endif // #else // #if (sc_VertexBlendingUseNormals)
    }
    #endif // #if (sc_VertexBlending)
}

vec4 sc_ObjectToScreen(vec4 objectPosition)
{
    #if (sc_RenderingSpace == 3)
    {
        return objectPosition;
    }
    #else // #if (sc_RenderingSpace == 3)
    {
        #if (sc_RenderingSpace == 2)
        {
            return sc_ViewProjectionMatrix * objectPosition;
        }
        #else // #if (sc_RenderingSpace == 2)
        {
            #if (sc_RenderingSpace == 1)
            {
                return sc_ModelViewProjectionMatrix * objectPosition;
            }
            #else // #if (sc_RenderingSpace == 1)
            {
                #if (sc_RenderingSpace == 4)
                {
                    return (sc_ModelViewMatrix * objectPosition) * vec4(1.0 / sc_Camera.aspect, 1.0, 1.0, 1.0);
                }
                #endif // #if (sc_RenderingSpace == 4)
            }
            #endif // #else // #if (sc_RenderingSpace == 1)
        }
        #endif // #else // #if (sc_RenderingSpace == 2)
    }
    #endif // #else // #if (sc_RenderingSpace == 3)
}

vec4 applyDepthAlgorithm(inout vec4 screenPosition)
{
    #if (sc_DepthBufferMode == 1)
    {
        float fCoefficient = 2.0 / log2(sc_Camera.clipPlanes.y + 1.0);
        screenPosition.z = ((log2(max(sc_Camera.clipPlanes.x, 1.0 + screenPosition.w)) * fCoefficient) - 1.0) * screenPosition.w;
    }
    #endif // #if (sc_DepthBufferMode == 1)
    return screenPosition;
}

sc_Vertex_t sc_LoadVertexAttributes()
{
    sc_Vertex_t v;
    v.position = position;
    #if ((sc_IsEditor == 1) && (SC_DISABLE_FRUSTUM_CULLING == 1))
    {
        v.position.x += sc_DisableFrustumCullingMarker;
    }
    #endif // #if ((sc_IsEditor == 1) && (SC_DISABLE_FRUSTUM_CULLING == 1))
    v.normal = normal;
    v.tangent = tangent.xyz;
    v.texture0 = texture0;
    v.texture1 = texture1;
    #if ((sc_StereoRendering_InstancedClipped == 1) && ((sc_RenderingSpace == 3) || (sc_RenderingSpace == 4)))
    {
        float stereoMatrixId = float(sc_StereoscopicMatrixIndex);
        v.texture0.y = (v.texture0.y * 0.5) + (0.5 - (stereoMatrixId * 0.5));
        v.texture1.y = (v.texture1.y * 0.5) + (0.5 - (stereoMatrixId * 0.5));
        v.position.y = (v.position.y * 0.5) + (0.5 - stereoMatrixId);
    }
    #endif // #if ((sc_StereoRendering_InstancedClipped == 1) && ((sc_RenderingSpace == 3) || (sc_RenderingSpace == 4)))
    return v;
}

void sc_ProcessVertex(inout sc_Vertex_t v)
{
    sc_Vertex_t param = v;
    sc_BlendVertex(param);
    v = param;
    sc_Vertex_t param_1 = v;
    sc_SkinVertex(param_1);
    v = param_1;
    vec4 param_2 = v.position;
    vec4 screenPosition = sc_ObjectToScreen(param_2);
    #if ((sc_RenderingSpace == 3) || (sc_RenderingSpace == 4))
    {
        varPos = screenPosition.xyz;
        varNormal = v.normal;
        varTangent = vec4(v.tangent.x, v.tangent.y, v.tangent.z, varTangent.w);
    }
    #else // #if ((sc_RenderingSpace == 3) || (sc_RenderingSpace == 4))
    {
        #if (sc_RenderingSpace == 2)
        {
            varPos = v.position.xyz;
            varNormal = v.normal;
            varTangent = vec4(v.tangent.x, v.tangent.y, v.tangent.z, varTangent.w);
        }
        #else // #if (sc_RenderingSpace == 2)
        {
            #if (sc_RenderingSpace == 1)
            {
                varPos = (sc_ModelMatrix * v.position).xyz;
                varNormal = sc_NormalMatrix * v.normal;
                vec3 _687 = sc_NormalMatrix * v.tangent;
                varTangent = vec4(_687.x, _687.y, _687.z, varTangent.w);
            }
            #endif // #if (sc_RenderingSpace == 1)
        }
        #endif // #else // #if (sc_RenderingSpace == 2)
    }
    #endif // #else // #if ((sc_RenderingSpace == 3) || (sc_RenderingSpace == 4))
    varNormal = normalize(varNormal);
    vec3 _694 = normalize(varTangent.xyz);
    varTangent = vec4(_694.x, _694.y, _694.z, varTangent.w);
    varTangent.w = tangent.w;
    varTex01 = vec4(v.texture0, v.texture1);
    #if (sc_ProjectiveShadowsReceiver)
    {
        vec4 param_3 = v.position;
    }
    #endif // #if (sc_ProjectiveShadowsReceiver)
    vec4 param_4 = screenPosition;
    vec4 _722 = applyDepthAlgorithm(param_4);
    screenPosition = _722;
    gl_Position = screenPosition;
}

#elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
#include <std2_shadows.glsl>
#include <std2.glsl>
precision mediump float;
precision highp int;

#ifndef sc_DepthBufferMode
#define sc_DepthBufferMode 0
#endif
#ifndef sc_SkinBonesCount
#define sc_SkinBonesCount 0
#endif
#ifndef SC_DISABLE_FRUSTUM_CULLING
#define SC_DISABLE_FRUSTUM_CULLING 0
#endif

uniform highp float sc_DisableFrustumCullingMarker;

varying highp vec3 varPos;
varying highp vec3 varNormal;
varying highp vec4 varTangent;
varying highp vec4 varTex01;

#endif // #elif defined FRAGMENT_SHADER // #if defined VERTEX_SHADER
