//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#pragma once

#include <required.glsl>


struct sc_PointLight_t {
    vec3 position;
    vec4 color;
};

struct sc_DirectionalLight_t {
    vec3 direction;
    vec4 color;
};

struct sc_AmbientLight_t {
    vec3 color;
    float intensity;

#ifndef SC_REMOVE_ENVMAP_FROM_AMBIENT_LIGHT_STRUCT  // 09/06/2018 HACK: Adreno GPUs can't handle textures in arrays of structs, some uniform values get corrupted, so we remove the textures from the ambient struct, since we don't use them anyway. This can be removed once we have no old uber shaders that sample envmaps from the ambient light struct.
    float weight;
    //vec3 position; // will be used for probes location

    // Members below this line are deprecated.

    vec3 sphericalHarmonics[9];

    vec3 rotation;
    float exposure;
    sampler2D diffuseEnvMap; // HACK: make sure textures are at the end of the struct. Google Pixel drivers seem to have a bug where they fail on some uniforms that come after textures in a struct!
    sampler2D specularEnvMap;
#endif
};

// The environment light is specified using loose uniforms to avoid device specific bugs with samplers in arrays/structs.
uniform sampler2D sc_EnvmapDiffuse;
uniform vec4 sc_EnvmapDiffuseSize;

uniform sampler2D sc_EnvmapSpecular;
uniform vec4 sc_EnvmapSpecularSize;

uniform vec3 sc_EnvmapRotation;
uniform float sc_EnvmapExposure;

// In some cases we support diffuse ambient light in SH coefficients. This replaces the diffuse envmap.
uniform vec3 sc_Sh[9];
uniform float sc_ShIntensity;

struct sc_AmbientLightCompatibility_t {
    vec3 color;
};

struct sc_Camera_t {
    // GPU_BUG_010 [LOOK-42347] 04/03/2019 Mali 400: This compiler forces all members of a struct to have the same precision, and assumes all members are used in a shader stage if any one member is used. We always use sc_Camera.position in the fragment shader, which forces precision of all members to mediump. Once we attempt to use sc_Camera.clipPlanes in the vertex shader, it creates highp uniforms for all members, which conflict with the mediump uniforms from the fragment shader. This means that even the "aspect" member conflicts with itself, even though it's not used at all, neither in vs nor fs. The solution is to force all members to fragment precision.
    sc_FragmentPrecision vec3 position;
    sc_FragmentPrecision float aspect;
    sc_FragmentPrecision vec2 clipPlanes; //x - near, y - far
};

uniform vec4 sc_Time;
#define sc_TimeElapsed (sc_Time.x)
#define sc_TimeDelta   (sc_Time.y)

uniform vec4 sc_GeometryInfo;
#define sc_PrimitiveCount (sc_GeometryInfo.x)
#define sc_InstanceCount  (sc_GeometryInfo.y)

#if defined(sc_StereoRendering_InstancedClipped) || defined(sc_StereoRendering_Multiview)

uniform mat4 sc_ModelViewProjectionMatrixArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ModelViewProjectionMatrixInverseArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ViewProjectionMatrixArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ViewProjectionMatrixInverseArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ModelViewMatrixArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ModelViewMatrixInverseArray[sc_StereoscopicEyeCount];
uniform mat3 sc_ViewNormalMatrixArray[sc_StereoscopicEyeCount];
uniform mat3 sc_ViewNormalMatrixInverseArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ProjectionMatrixArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ProjectionMatrixInverseArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ViewMatrixArray[sc_StereoscopicEyeCount];
uniform mat4 sc_ViewMatrixInverseArray[sc_StereoscopicEyeCount];

/* Since gl_InstanceID won't be available in fragment shaders,
 * varStereoscopicEyeID will be assigned in clipPosition() (in gles20std_vs.glsl)
 * and used instead of sc_StereoscopicEyeID macro in fragment shaders.
 */
#ifdef sc_EnableFeatureLevelES3
flat varying int varStereoscopicEyeID;
#else
varying float varStereoscopicEyeID;
#endif

#ifdef sc_StereoRendering_InstancedClipped
#if sc_StereoRendering_IsClipDistanceEnabled
// GLES20ForwardRenderer only enables a single clip distance slot.
#define sc_ClipDistance0 gl_ClipDistance[0]
#else
varying float varClipDistance;
#define sc_ClipDistance0 varClipDistance
#endif // sc_StereoRendering_IsClipDistanceEnabled
#endif // sc_StereoRendering_InstancedClipped

#ifdef FRAGMENT_SHADER
#define sc_StereoscopicMatrixIndex int(varStereoscopicEyeID)
#else
#define sc_StereoscopicMatrixIndex sc_StereoscopicEyeID
#endif

#define sc_ModelViewProjectionMatrix        (sc_ModelViewProjectionMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ModelViewProjectionMatrixInverse (sc_ModelViewProjectionMatrixInverseArray[sc_StereoscopicMatrixIndex])
#define sc_ViewProjectionMatrix             (sc_ViewProjectionMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ViewProjectionMatrixInverse      (sc_ViewProjectionMatrixInverseArray[sc_StereoscopicMatrixIndex])
#define sc_ModelViewMatrix                  (sc_ModelViewMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ModelViewMatrixInverse           (sc_ModelViewMatrixInverseArray[sc_StereoscopicMatrixIndex])
#define sc_ViewNormalMatrix                 (sc_ViewNormalMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ViewNormalMatrixInverse          (sc_ViewNormalMatrixInverseArray[sc_StereoscopicMatrixIndex])
#define sc_ProjectionMatrix                 (sc_ProjectionMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ProjectionMatrixInverse          (sc_ProjectionMatrixInverseArray[sc_StereoscopicMatrixIndex])
#define sc_ViewMatrix                       (sc_ViewMatrixArray[sc_StereoscopicMatrixIndex])
#define sc_ViewMatrixInverse                (sc_ViewMatrixInverseArray[sc_StereoscopicMatrixIndex])

#else

uniform mat4 sc_ModelViewProjectionMatrix;
uniform mat4 sc_ModelViewProjectionMatrixInverse;
uniform mat4 sc_ViewProjectionMatrix;
uniform mat4 sc_ViewProjectionMatrixInverse;
uniform mat4 sc_ModelViewMatrix;
uniform mat4 sc_ModelViewMatrixInverse;
uniform mat3 sc_ViewNormalMatrix;
uniform mat3 sc_ViewNormalMatrixInverse;
uniform mat4 sc_ProjectionMatrix;
uniform mat4 sc_ProjectionMatrixInverse;
uniform mat4 sc_ViewMatrix;
uniform mat4 sc_ViewMatrixInverse;

#endif

uniform mat4 sc_ModelMatrix;
uniform mat4 sc_ModelMatrixInverse;

uniform mat3 sc_NormalMatrix;
uniform mat3 sc_NormalMatrixInverse;

uniform sampler2D sc_ScreenTexture;
uniform vec4 sc_ScreenTextureSize;
uniform sc_Camera_t sc_Camera;

#ifdef sc_PointLightsCount
uniform sc_PointLight_t sc_PointLights[sc_PointLightsCount];
#endif

#ifdef sc_DirectionalLightsCount
uniform sc_DirectionalLight_t sc_DirectionalLights[sc_DirectionalLightsCount];
#endif

#ifdef sc_AmbientLightsCount
uniform sc_AmbientLight_t sc_AmbientLights[sc_AmbientLightsCount];
#endif

// backward compatibility
sc_AmbientLightCompatibility_t getAmbientLightCompatibility() {
    sc_AmbientLightCompatibility_t ambient;
#ifdef sc_AmbientLightsCount
    ambient.color = sc_AmbientLights[0].color * sc_AmbientLights[0].intensity;
#else
    ambient.color = vec3(0.0);
#endif
    return ambient;
}
#define sc_AmbientLight (getAmbientLightCompatibility())

#if sc_ExporterVersion < 74
#include <std_shadows.glsl>  // HACK 05/15/2018: some particle shaders were shipped in public studio that did not directly include this, but relied on an old hack that included std_shadows.glsl in std.glsl. Once all those old assets die, we can remove this. Remove related hack from std_shadows.glsl.
#endif

//
// Stereo texture sampling
//

// sc_StereoClampTextureCoords(uv) makes sure that in stereo mode,
// the left and right parts of the background texture are sampled exclusively.
// For example, consider a blurring algorithm that samples and mixes neighbor pixels.
// For the "instanced-clipped" approach, a default implementation would contain artifacts,
// since as a result, one part includes information from the other one on its bottom/upper edges.
//
// Visualization:
// x - current pixel
// o - neighbor pixels
//
// .-------.< 1
// |       |
// |    o  |
// |       |
// |    x  |   L
// .-------.< 0.5
// |    o  |   R
// |       |
// |       |
// |       |
// .-------.< 0
//
vec2 sc_StereoClampTextureCoords(vec2 uv) {
    #if defined(sc_StereoRendering_InstancedClipped)

    // Contains precalculated value = 1.0/height.
    // For stereo rendering, we should make sure that sampling of the background texture
    // is exclusive for its logical parts of stereoscopic eyes.
    float pixelHeight = sc_ScreenTextureSize.w;

    uv.y = sc_StereoscopicMatrixIndex == 0
         ? max(uv.y, 0.5 + pixelHeight)
         : min(uv.y, 0.5 - pixelHeight);

    #endif // sc_StereoRendering_InstancedClipped
    return uv;
}

// sc_SampleScreenTexture(uv) is a special function for sampling the screen textures in stereo mode.
// See sc_StereoClampTextureCoords() for more info.
vec4 sc_SampleScreenTexture(vec2 uv) {
    // Notice, the extra definition is needed for GLShaderSourceBuilder to avoid recursion,
    // as it substitutes "texture2D(sc_ScreenTexture," with "sc_SampleScreenTexture(".
    #define stereoScreenTexture sc_ScreenTexture
    return texture2D(stereoScreenTexture, sc_StereoClampTextureCoords(uv));
    #undef stereoScreenTexture
}

// sc_StereoClampMapTextureCoords(uv) maps passed uv coordinates to a current stereo eye in stereo mode.
// For the "instanced-clipped" approach, a default implementation would contain artifacts.
//
// Visualization:
// x - sample location
// a - sample location re-mapped in stereo mode for the left stereoscopic eye.
// b - sample location re-mapped in stereo mode for the right stereoscopic eye.
//
//                      .-------.< 1
//                      |    a  |
// .-------.< 1         |       |
// |    x  |            |       |
// |       |            |       |   L
// |       |      =>    .-------.< 0.5
// |       |            |    b  |   R
// |       |            |       |
// .-------.< 0         |       |
//                      |       |
//                      .-------.< 0
//
vec2 sc_StereoClampMapTextureCoords(vec2 uv) {
    #if defined(sc_StereoRendering_InstancedClipped)

    float stereoMatrixId = float(sc_StereoscopicMatrixIndex);
    uv.y = (uv.y * 0.5) + (0.5 - stereoMatrixId * 0.5);

    #endif // sc_StereoRendering_InstancedClipped
    return sc_StereoClampTextureCoords(uv);
}

// A more comprehensive alias for lens shaders.
#define sc_SampleScreenTextureForCurrentEye sc_SampleMapScreenTexture

// sc_SampleMapScreenTexture(uv) is a special function for sampling the screen textures in stereo mode,
// that maps passed uv coordinates to a current stereoscopic eye region of a high texture.
vec4 sc_SampleMapScreenTexture(vec2 uv) {
    // Notice, the extra definition is needed for GLShaderSourceBuilder to avoid recursion,
    // as it substitutes "texture2D(sc_ScreenTexture," with "sc_SampleMapScreenTexture(".
    #define stereoScreenTexture sc_ScreenTexture
    return texture2D(stereoScreenTexture, sc_StereoClampMapTextureCoords(uv));
    #undef stereoScreenTexture
}
