//-----------------------------------------------------------------------
// Copyright (c) 2018 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_vs.metal"
#include "metalstd_fs.metal"
#else
#include <metal_std.metal>
#include <metalstd_vs.metal>
#include <metalstd_fs.metal>
#endif

struct Vertex {
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct ColorInOut {
    float4 position [[position]];
    half2 texture0;
};

struct Uniforms
{
    float3x3 inputTextureTransform;
};

constant bool SWAP_R_B_CHANNELS [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constexpr half4 colorCoeff = half4(0.299, 0.587, 0.114, 0.0);

// vertex shader function
vertex ColorInOut vs_main(Vertex vtx [[stage_in]],
                          constant Uniforms& uniforms [[ buffer(2) ]])
{
    ColorInOut out;
    out.position = float4(vtx.position, 0.0, 1.0);
    out.texture0 = half2((uniforms.screenTextureTransform * float3(vtx.texture0, 1.0)).xy);
    return out;
}

// fragment shader function
fragment half4 fs_main(ColorInOut in [[stage_in]],
                       texture2d<half> inputTexture,
                       sampler inputTextureSampler)
{
    half4 color;
    
    if (SWAP_R_B_CHANNELS) {
        color = inputTexture.sample(inputTextureSampler, float2(in.texture0)).bgra;
    } else {
        color = inputTexture.sample(inputTextureSampler, float2(in.texture0));
    }
    
    half grayscale = dot(color, colorCoeff);
    return half4(grayscale);
};

