//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "metal_std.metal"
#else
#include <metal_std.metal>
#endif

struct Vertex
{
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct Uniforms
{
    float3x3 cameraTextureTransform;
    float4 leftSampleOffsetSize;
    float4 rightSampleOffsetSize;
};

struct Varyings
{
    float4 position[[position]];
    float varLerp;
    float2 varLeftSampleCoord;
    float2 varRightSampleCoord;
};

// vertex shader function
vertex Varyings vs_main(Vertex vtx [[ stage_in ]],
                          constant Uniforms& uniforms [[ buffer(2) ]])
{
    Varyings var;
    float3x3 leftSampleTransform = uniforms.cameraTextureTransform;
    // Scale by sample's width and height.
    leftSampleTransform[0][0] *= uniforms.leftSampleOffsetSize.z;
    leftSampleTransform[1][1] *= uniforms.leftSampleOffsetSize.w;
    
    float3x3 rightSampleTransform = uniforms.cameraTextureTransform;
    // Scale by sample's width and height.
    rightSampleTransform[0][0] *= uniforms.rightSampleOffsetSize.z;
    rightSampleTransform[1][1] *= uniforms.rightSampleOffsetSize.w;
    
    // Transform from [0, 1] to [-1, 1] and apply transformations.
    float2 rangedTexCoord = vtx.texture0 * float2(2) - float2(1);
    float3 leftSampleCoord = float3(rangedTexCoord, 1) * leftSampleTransform;
    float3 rightSampleCoord = float3(rangedTexCoord, 1) * rightSampleTransform;
    
    // Factor for lerping samples.
    var.varLerp = 1.0 - vtx.texture0.x;

    // Apply offset and ransform back from [-1, 1] to [0, 1].
    var.varLeftSampleCoord = (leftSampleCoord.xy + uniforms.leftSampleOffsetSize.xy) * 0.5 + 0.5;
    var.varRightSampleCoord = (rightSampleCoord.xy + uniforms.rightSampleOffsetSize.xy) * 0.5 + 0.5;

    var.position = float4(vtx.position, 0.0, 1.0);
    
    var.position.y = -vtx.position.y;

    return var;
}

fragment half4 fs_main(Varyings in [[stage_in]],texture2d<half> cameraTexture,
                       sampler cameraTextureSampler)
{
    half3 leftSample = cameraTexture.sample(cameraTextureSampler, in.varLeftSampleCoord).rgb;
    half3 rightSample = cameraTexture.sample(cameraTextureSampler, in.varRightSampleCoord).rgb;
    half3 lerpedColor = mix(leftSample, rightSample, half(in.varLerp));
    return half4(lerpedColor, 1.0);
}
