//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#define BASECOLOR baseColor

#define BASETEXNAME baseTex
#define BASETEXNAMESAMPLER baseTexSampler
#define BASETEXNAMETRANSFORM baseTexTransform

#define MASKTEXNAME opacityTex
#define MASKTEXNAMESAMPLER opacityTexSampler
#define MASKTEXNAMETRANSFORM opacityTexTransform

#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_vs.metal"
#include "metalstd_fs.metal"
#else
#include <metal_std.metal>
#include <metalstd_vs.metal>
#include <metalstd_fs.metal>
#endif

typedef float2 vec2;
typedef float3 vec3;
typedef float4 vec4;
typedef float4x4 mat4;



struct FSUniforms
{
  float4 BASECOLOR;
    TEXUNIFORMS(baseTex)
    TEXUNIFORMS(opacityTex)
};


constant bool NOTEXTURE [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant bool HASMAINTEXTURE  = (NOTEXTURE==false);
constant bool NOMASK [[function_constant(FUNCONST_VS_USER_BIAS+1)]];
constant bool HASMASK = (NOMASK==false);
SC_USE_BLOCK_BIAS(baseTex,FUNCONST_VS_USER_BIAS,2+5*0)
SC_USE_BLOCK_BIAS(opacityTex,FUNCONST_VS_USER_BIAS,2+5*1)

constant bool sc_BlendMode_Custom [[function_constant(FUNCONST_VS_USER_BIAS+30)]];
constant bool NeedFramebufferFetch = sc_BlendMode_Custom;
NEED_SCREENTEXTURE();

#if defined(CMDLINE)
#include "includes/blend_modes.metal"
#else
#include <includes/blend_modes.metal>
#endif


VSMAIN()
{
  sc_Vertex_t v = sc_LoadVertexAttributes(vtx);
  sc_ProcessVertex(v);
}

// fragment shader function
fragment half4 fs_main(Varyings in [[stage_in]],
                       constant FSUniforms& fsUniforms [[ buffer(2) ]],
                       DECLARETEXTUREOPTIONAL(BASETEXNAME,BASETEXNAMESAMPLER,HASMAINTEXTURE),
                       DECLARETEXTUREOPTIONAL(MASKTEXNAME,MASKTEXNAMESAMPLER,HASMASK),
                       DECLARETEXTUREOPTIONAL(sc_ScreenTexture,sc_ScreenTextureSampler,needScreenTexture)
                       OPTIONAL_FRAMEBUFFERFETCH_PARAM()
)
{
    OPTIONAL_FRAMEBUFFERFETCH_VAR();

    half4 result = half4(fsUniforms.BASECOLOR);
  
  if(ENABLE_VERTEX_COLOR_BASE && HASCOLORATTRIB )
  {
    result *= half4(in.varColor);
  }
  
  if(HASMAINTEXTURE)
  {
    SAMPLE_TEX(baseTex,in.varTex0, 0.0);
    result *= baseTexSample;
  }
  
  if(HASMASK)
  {
    SAMPLE_TEX(opacityTex,in.varTex1, 0.0);
    result.a *= opacityTexSample.r;
  }
  
    // Blending
    if (sc_BlendMode_Custom)
    {
        result = (half4)applyCustomBlend((float4)result,frameBufferColor,sc_ScreenTexture,sc_ScreenTextureSampler, in.varScreenPos);
    }
    else if(sc_BlendMode_MultiplyOriginal)
    {
        result.rgb = mix(half3(1.0), result.rgb, result.a);
    }
    else if(sc_BlendMode_Screen)
    {
        result.rgb = result.rgb * result.a;
    }

    return result;
};
