//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "metal_std.metal"
#else
#include <metal_std.metal>
#endif

constant int sc_GaussianFilterDataLength [[function_constant(FUNCONST_FS_BIAS+0)]];
constant bool sc_GaussianFilterVerticalPass[[function_constant(FUNCONST_FS_BIAS+1)]];

constant int maxGaussianFilterLen = 20;
struct Vertex
{
  float2 position [[attribute(0)]];
};

struct Uniforms
{
  float gaussianFilterWeights[maxGaussianFilterLen];
  float gaussianFilterOffsets[maxGaussianFilterLen];
};

struct Varyings {
  float4 gl_Position [[position]];
  float2 varTexCoords;
};

// vertex shader function
vertex Varyings vs_main(Vertex vtx [[ stage_in ]],
                          constant Uniforms& uniforms [[ buffer(2) ]])
{
  Varyings out;
  out.varTexCoords = vtx.position * 0.5 + 0.5;
  out.gl_Position = float4(vtx.position, 0.0, 1.0);
  return out;
}

fragment half4 fs_main(Varyings in [[stage_in]],
                       constant Uniforms& uniforms [[ buffer(2) ]],
                       texture2d<half> targetTexture,
                       sampler targetTextureSampler)
{
   half4 result = targetTexture.sample(targetTextureSampler, in.varTexCoords) * uniforms.gaussianFilterWeights[0];
    
    for (int index = 1; index < sc_GaussianFilterDataLength; index++) {
        float4 uv = in.varTexCoords.xyxy;
        if(sc_GaussianFilterVerticalPass)
        {
                uv.y += uniforms.gaussianFilterOffsets[index];
                uv.w -= uniforms.gaussianFilterOffsets[index];
        }
        else
        {
                uv.x += uniforms.gaussianFilterOffsets[index];
                uv.z -= uniforms.gaussianFilterOffsets[index];
        } // sc_GaussianFilterVerticalPass
        result += (targetTexture.sample(targetTextureSampler, uv.xy) + targetTexture.sample(targetTextureSampler, uv.zw)) * uniforms.gaussianFilterWeights[index];
    }
    
    return result;
}
