//-----------------------------------------------------------------------
// Copyright (c) 2018 Snap Inc.
//-----------------------------------------------------------------------
#ifndef METALSTD_METAL
#define METALSTD_METAL
#include <metal_stdlib>
using namespace metal;

#if defined(CMDLINE)
#include "funconst.metal"
#else
#include <funconst.metal>
#endif

struct sc_Camera_t {
    float3 position;
    float aspect;
    float2 clipPlanes; //x - near, y - far
};

struct sc_PointLight_t {
  float3 position;
  float4 color;
};

struct sc_DirectionalLight_t {
  float3 direction;
  float4 color;
};

struct sc_AmbientLight_t {
  float3 color;
  float intensity;
  
//#ifndef SC_REMOVE_ENVMAP_FROM_AMBIENT_LIGHT_STRUCT  // 09/06/2018 HACK: Adreno GPUs can't handle textures in arrays of structs, some uniform values get corrupted, so we remove the textures from the ambient struct, since we don't use them anyway. This can be removed once we have no old uber shaders that sample envmaps from the ambient light struct.
  float weight;
  //vec3 position; // will be used for probes location
  
  // Members below this line are deprecated.
  
  float3 sphericalHarmonics[9];
  
  float3 rotation;
  float exposure;
  //sampler2D diffuseEnvMap; // HACK: make sure textures are at the end of the struct. Google Pixel drivers seem to have a bug where they fail on some uniforms that come after textures in a struct!
  //sampler2D specularEnvMap;
};


// keep in sync with Rendering/TextureDefines.h
enum TextureDefinesWrapMode {
    Hardware = -1,
    SoftwareClampToEdge,
    SoftwareRepeat,
    SoftwareMirroredRepeat,
    SoftwareClampToBorder
};
    enum SamplerDescriptorWrapMode {
        HardwareDesc = -1,
        SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE=0,
        SOFTWARE_WRAP_MODE_REPEAT=1,
        SOFTWARE_WRAP_MODE_MIRRORED_REPEAT=2,
        SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER=3
        };

// keep in sync with Primitives/RenderingSpace.h
enum RenderingSpace
{
    ObjectSpace = 1,
    WorldSpace = 2,
    ScreenSpace = 3,
    ScreenSpaceMV = 4
};

  enum AmbientLightMode
  {
    Constant = 1,
    SphericalHarmonics = 2,
    EnvironmentMap = 3
  };
      

      
constant int SC_DEVICE_CLASS_A [[function_constant(FUNCONST_GLOBAL_BIAS+0)]];
constant int SC_DEVICE_CLASS_B [[function_constant(FUNCONST_GLOBAL_BIAS+1)]];
constant int SC_DEVICE_CLASS_C [[function_constant(FUNCONST_GLOBAL_BIAS+2)]];
constant int SC_DEVICE_CLASS_D [[function_constant(FUNCONST_GLOBAL_BIAS+3)]];
constant int SC_DEVICE_CLASS [[function_constant(FUNCONST_GLOBAL_BIAS+4)]];
constant bool sc_FramebufferFetch [[function_constant(FUNCONST_GLOBAL_BIAS+5)]];
constant bool NoFramebufferFetch = (sc_FramebufferFetch==false);

float4 fixProjectionForMetal(float4 pos)
{
  pos.z = pos.z*0.5+0.5;
  return pos;
}
    
#if __METAL_IOS__
#define PLATFORM_WITH_FRAMEBUFFER_FETCH 1
#define NEED_SCREENTEXTURE() constant bool needScreenTexture = false;
#else
#define NEED_SCREENTEXTURE() constant bool needScreenTexture = true;
#define PLATFORM_WITH_FRAMEBUFFER_FETCH 0
#endif
      
      
#endif
