//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#ifndef METALSTD_FS
#define METALSTD_FS


#define TEXTUREPARAM(texName) texture2d<half> texName, sampler texName##Sampler
#define TEXTUREPARAMOPTIONAL(texName,nameopt) texture2d<half> texName[[function_constant(nameopt)]], sampler texName##Sampler[[function_constant(nameopt)]]
#define DECLARETEXTURE(texName,texNameSampler) texture2d<half> texName, sampler texNameSampler
#define DECLARETEXTUREOPTIONAL(texName,texNameSampler,nameopt) texture2d<half> texName[[function_constant(nameopt)]], sampler texNameSampler[[function_constant(nameopt)]]

#define TEXTURE2DARRAYPARAM(texName) texture2d_array<half> texName, sampler texName##Sampler
#define TEXTURE2DARRAYPARAMOPTIONAL(texName,nameopt) texture2d_array<half> texName[[function_constant(nameopt)]], sampler texName##Sampler[[function_constant(nameopt)]]
#define DECLARETEXTURE2DARRAY(texName,texNameSampler) texture2d_array<half> texName, sampler texNameSampler
#define DECLARETEXTURE2DARRAYOPTIONAL(texName,texNameSampler,nameopt) texture2d_array<half> texName[[function_constant(nameopt)]], sampler texNameSampler[[function_constant(nameopt)]]

#define TEXTURE3DPARAM(texName) texture3d<half> texName, sampler texName##Sampler
#define TEXTURE3DPARAMOPTIONAL(texName,nameopt) texture3d<half> texName[[function_constant(nameopt)]], sampler texName##Sampler[[function_constant(nameopt)]]
#define DECLARETEXTURE3D(texName,texNameSampler) texture3d<half> texName, sampler texNameSampler
#define DECLARETEXTURE3DOPTIONAL(texName,texNameSampler,nameopt) texture3d<half> texName[[function_constant(nameopt)]], sampler texNameSampler[[function_constant(nameopt)]]

#define TEXTURECUBEPARAM(texName) texturecube<half> texName, sampler texName##Sampler
#define TEXTURECUBEPARAMOPTIONAL(texName,nameopt) texturecube<half> texName[[function_constant(nameopt)]], sampler texName##Sampler[[function_constant(nameopt)]]
#define DECLARETEXTURECUBE(texName,texNameSampler) texturecube<half> texName, sampler texNameSampler
#define DECLARETEXTURECUBEOPTIONAL(texName,texNameSampler,nameopt) texturecube<half> texName[[function_constant(nameopt)]], sampler texNameSampler[[function_constant(nameopt)]]


#define CLAMP_UV(uv, uvMin, uvMax, useClampToBorder, borderClampFactor) {\
    float clampedUv = clamp(uv, uvMin, uvMax);\
\
    if (useClampToBorder) {\
        borderClampFactor *= step(abs(uv - clampedUv), 0.00001);\
    }\
\
    uv = clampedUv;\
}

#define SOFTWARE_WRAP(uv, softwareWrapMode, useClampToBorder, clampToBorderFactor) {\
    if (softwareWrapMode == SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE || softwareWrapMode == SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER) {\
        CLAMP_UV(uv, 0.0, 1.0, useClampToBorder, clampToBorderFactor); \
    } else if (softwareWrapMode == SOFTWARE_WRAP_MODE_REPEAT) {\
        uv = fract(uv); \
    } else if (softwareWrapMode == SOFTWARE_WRAP_MODE_MIRRORED_REPEAT) {\
        float uvFract = fract(uv);\
        float uvInt = uv - uvFract;\
        float uvOdd = step(0.25, fract(uvInt * 0.5));\
        uv = mix(uvFract, 1.0 - uvFract, saturate(uvOdd));\
    }\
}


half4 sampleTextureSimple(texture2d<half> texName, sampler texNameSampler, float2 uv)
{
    return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y));
}

half4 sampleTexture(texture2d<half> texName, sampler texNameSampler, float2 uv, float biasValue)
{
  return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y),bias(biasValue));
}

half4 sampleTextureLevel(texture2d<half> texName, sampler texNameSampler, float2 uv, float levelVal)
{
    return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y),level(levelVal));
}


half4 sampleTextureSimple(texture2d_array<half> texName, sampler texNameSampler, float2 uv, uint index)
{
    return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y), index);
}

half4 sampleTexture(texture2d_array<half> texName, sampler texNameSampler, float2 uv, uint index, float biasValue)
{
  return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y), index, bias(biasValue));
}

half4 sampleTextureLevel(texture2d_array<half> texName, sampler texNameSampler, float2 uv, uint index, float levelVal)
{
    return texName.sample(texNameSampler,float2(uv.x,1.0-uv.y), index, level(levelVal));
}


half4 sampleTextureSimple(texture3d<half> texName, sampler texNameSampler, float3 uvw)
{
    return texName.sample(texNameSampler,float3(uvw.x,1.0-uvw.y,uvw.z));
}

half4 sampleTexture(texture3d<half> texName, sampler texNameSampler, float3 uvw, float biasValue)
{
  return texName.sample(texNameSampler,float3(uvw.x,1.0-uvw.y,uvw.z),bias(biasValue));
}

half4 sampleTextureLevel(texture3d<half> texName, sampler texNameSampler, float3 uvw, float levelVal)
{
    return texName.sample(texNameSampler,float3(uvw.x,1.0-uvw.y,uvw.z),level(levelVal));
}


half4 sampleTextureSimple(texturecube<half> texName, sampler texNameSampler, float3 uvw)
{
    return texName.sample(texNameSampler,float3(uvw.x,-uvw.y,uvw.z));
}

half4 sampleTexture(texturecube<half> texName, sampler texNameSampler, float3 uvw, float biasValue)
{
  return texName.sample(texNameSampler,float3(uvw.x,-uvw.y,uvw.z),bias(biasValue));
}

half4 sampleTextureLevel(texturecube<half> texName, sampler texNameSampler, float3 uvw, float levelVal)
{
    return texName.sample(texNameSampler,float3(uvw.x,-uvw.y,uvw.z),level(levelVal));
}


#define TEXUNIFORMS(NAME) \
float3x3 NAME##Transform;\
float4 NAME##UvMinMax;\
float4 NAME##BorderColor; \

#define TEX2DARRAYUNIFORMS(NAME) \
float3x3 NAME##Transform;\
float4 NAME##UvMinMax;\
float4 NAME##BorderColor; \
float4 NAME##Size;\
float2 NAME##ArrayCount;\

#define TEX3DUNIFORMS(NAME) \
float4 NAME##UvMinMax;\
float4 NAME##BorderColor; \
float2 NAME##WMinMax;\
float4 NAME##Size;\
float2 NAME##Depth;\

#define TEXCUBEUNIFORMS(NAME) \
float4 NAME##Size;\

#define SC_USE_BLOCK_BIAS(NAME,SUPERBIAS,CONSTBIAS) \
constant bool SC_USE_UV_TRANSFORM_##NAME [[function_constant(SUPERBIAS+CONSTBIAS+0)]]; \
constant int SC_SOFTWARE_WRAP_MODE_U_##NAME [[function_constant(UBER_CONSTS_SURFPROP_BIAS+CONSTBIAS+1)]]; \
constant int SC_SOFTWARE_WRAP_MODE_V_##NAME [[function_constant(UBER_CONSTS_SURFPROP_BIAS+CONSTBIAS+2)]]; \
constant bool SC_USE_UV_MIN_MAX_##NAME [[function_constant(UBER_CONSTS_SURFPROP_BIAS+CONSTBIAS+3)]]; \
constant bool SC_USE_CLAMP_TO_BORDER_##NAME [[function_constant(UBER_CONSTS_SURFPROP_BIAS+CONSTBIAS+4)]];


#define QUANTIZE_UV(A) A

half4 SAMPLE_TEXTURE(texture2d<half> texName, sampler texNameSampler, float2 uvIn, bool useUvTransform, float3x3 uvTransform, int softwareWrapModeU, int softwareWrapModeV, bool useUvMinMax, float4 uvMinMax, bool useClampToBorder, float4 borderColor, float mipBias) {
    
    half4 tex;
    float2 uvTemp = uvIn;
    float clampToBorderFactor = 1.0;

    SOFTWARE_WRAP(uvTemp.x, softwareWrapModeU, useClampToBorder && !useUvMinMax, clampToBorderFactor);\
    SOFTWARE_WRAP(uvTemp.y, softwareWrapModeV, useClampToBorder && !useUvMinMax, clampToBorderFactor);\

    if (useUvMinMax) {
        CLAMP_UV(uvTemp.x, uvMinMax.x, uvMinMax.z, useClampToBorder, clampToBorderFactor);\
        CLAMP_UV(uvTemp.y, uvMinMax.y, uvMinMax.w, useClampToBorder, clampToBorderFactor);\
    }

    if (useUvTransform) {
        uvTemp = (uvTransform * float3(uvTemp, 1.0)).xy;
    }

    uvTemp = QUANTIZE_UV(uvTemp);

    tex = sampleTexture(texName,texNameSampler, uvTemp, mipBias);

    if (useClampToBorder) {
        tex = mix((half4)borderColor, tex, (half)clampToBorderFactor);
    }
    return tex;
}

#define SAMPLE_TEX(NAME,UV,MIP_BIAS) half4 NAME##Sample = SAMPLE_TEXTURE(NAME,NAME##Sampler,UV,SC_USE_UV_TRANSFORM_##NAME,fsUniforms.NAME##Transform,SC_SOFTWARE_WRAP_MODE_U_##NAME,SC_SOFTWARE_WRAP_MODE_V_##NAME, SC_USE_UV_MIN_MAX_##NAME, fsUniforms.NAME##UvMinMax, SC_USE_CLAMP_TO_BORDER_##NAME, fsUniforms.NAME##BorderColor ,MIP_BIAS)


half4 SAMPLE_TEXTURE_2D_ARRAY(texture2d_array<half> texName, sampler texNameSampler, float2 uvIn, uint index, bool useUvTransform, float3x3 uvTransform, float mipBias) {
    
    half4 tex;
    float2 uvTemp = uvIn;

    if (useUvTransform) {
        uvTemp = (uvTransform * float3(uvTemp, 1.0)).xy;
    }

    uvTemp = QUANTIZE_UV(uvTemp);

    tex = sampleTexture(texName,texNameSampler, uvTemp, index, mipBias);

    return tex;
}

#define SAMPLE_TEX_2D_ARRAY(NAME,UVW,MIP_BIAS) half4 NAME##Sample = SAMPLE_TEXTURE_2D_ARRAY(NAME,NAME##Sampler,UVW.xy,uint(UVW.z),SC_USE_UV_TRANSFORM_##NAME,fsUniforms.NAME##Transform,MIP_BIAS)

half4 SAMPLE_TEXTURE_3D(texture3d<half> texName, sampler texNameSampler, float3 uvwIn, float mipBias) {
    
    half4 tex;
    float3 uvwTemp = uvwIn;

    uvwTemp = QUANTIZE_UV(uvwTemp);

    tex = sampleTexture(texName, texNameSampler, uvwTemp, mipBias);

    return tex;
}

#define SAMPLE_TEX_3D(NAME,UVW,MIP_BIAS) half4 NAME##Sample = SAMPLE_TEXTURE_3D(NAME,NAME##Sampler,UVW,MIP_BIAS)


half4 SAMPLE_TEXTURE_CUBE(texturecube<half> texName, sampler texNameSampler, float3 uvwIn, float mipBias) {
    
    half4 tex;
    float3 uvwTemp = uvwIn;

    uvwTemp = QUANTIZE_UV(uvwTemp);

    tex = sampleTexture(texName, texNameSampler, uvwTemp, mipBias);

    return tex;
}

#define SAMPLE_TEX_CUBE(NAME,UVW,MIP_BIAS) half4 NAME##Sample = SAMPLE_TEXTURE_CUBE(NAME,NAME##Sampler,UVW,MIP_BIAS)


#define TEXTUREASPARAM(NAME) NAME, NAME##Sampler
#define TEXTUREASPARAMWITHSAMPLER(NAME,SAMPLER) NAME, SAMPLER

half4 getFramebufferColor(half4 fragData,DECLARETEXTUREOPTIONAL(ScreenTexture,ScreenTextureSampler,NoFramebufferFetch), float4 varScreenPos[[function_constant(NoFramebufferFetch)]] ) {
    if( sc_FramebufferFetch )
    {
        return fragData;
    }
    else
    {
        return sampleTextureSimple(ScreenTexture,ScreenTextureSampler,varScreenPos.xy / varScreenPos.w * 0.5 + 0.5);
    }
}





/*
include <std_texture.glsl>

vec3 unpackNormal(sampler2D tex, vec2 coords) {
    vec3 N = normalize(varNormal);
    vec3 T = normalize(varTangent);
    vec3 B = normalize(cross(varNormal, varTangent) * varBitangentSign);
    mat3 TBN = mat3(T, B, N);
    vec3 nm = texture2D(tex, coords).xyz * 2.0 - vec3(1.0);
    return normalize(TBN * normalize(nm));
}
*/


#if PLATFORM_WITH_FRAMEBUFFER_FETCH
#define OPTIONAL_FRAMEBUFFERFETCH_PARAM() ,float4 frameBufferColor [[function_constant(sc_FramebufferFetch),color(0)]]
#define OPTIONAL_FRAMEBUFFERFETCH_VAR()
#else
#define OPTIONAL_FRAMEBUFFERFETCH_PARAM()
#define OPTIONAL_FRAMEBUFFERFETCH_VAR() float4 frameBufferColor;
#endif

#endif //METALSTD_FS
