//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#ifndef METALSTD_SHADOWS
#define METALSTD_SHADOWS

#if defined(CMDLINE)
#include "metalstd_fs.metal"
#else
#include <metalstd_fs.metal>
#endif

constant bool sc_BlendMode_Normal [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+0)]];
constant bool sc_BlendMode_AlphaToCoverage [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+1)]];
constant bool sc_BlendMode_PremultipliedAlpha [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+3)]];
constant bool sc_BlendMode_AddWithAlphaFactor [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+4)]];
constant bool sc_BlendMode_AlphaTest [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+5)]];
constant bool sc_BlendMode_Multiply [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+6)]];
constant bool sc_BlendMode_MultiplyOriginal [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+7)]];
constant bool sc_BlendMode_ColoredGlass [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+8)]];
constant bool sc_BlendMode_Add [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+9)]];
constant bool sc_BlendMode_Screen [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+11)]];
constant bool sc_BlendMode_Min [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+12)]];
constant bool sc_BlendMode_Max [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+13)]];
constant bool sc_BlendMode_Disabled [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+14)]];
constant bool sc_BlendMode_PremultipliedAlphaHardware [[function_constant(METALSTD_SHADOWS_CONSTS_BIAS+15)]];

struct VSShadowsUniforms
{
  float4x4 sc_ProjectorMatrix;
};

struct FSShadowsUniforms
{
  float4 sc_ShadowColor;
  float sc_ShadowDensity;
};

half getShadowAlpha(half4 color, float alphaTestThreshold) {
  half shadowAlpha = 1.0;
  if (sc_BlendMode_Normal || sc_BlendMode_AlphaToCoverage || sc_BlendMode_PremultipliedAlphaHardware)
  {
    shadowAlpha = color.a;
  }
  else if (sc_BlendMode_PremultipliedAlpha)
  {
    shadowAlpha = clamp(color.a * 2.0h, 0.0h, 1.0h);
  }
  //else if(sc_BlendMode_AddWithAlphaFactor)
  //{
  //  shadowAlpha = clamp(dot(color.rgb, float3(color.a)), 0.0, 1.0);
  //}
  else if(sc_BlendMode_AlphaTest)
  {
    shadowAlpha = color.a < alphaTestThreshold ? 0.0h : 1.0h;
  }
  else if(sc_BlendMode_Multiply)
  {
    shadowAlpha = (1.0 - dot(color.rgb, half3(0.33333))) * color.a;
  }
  else if(sc_BlendMode_MultiplyOriginal)
  {
    shadowAlpha = (1.0 - clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h)) * color.a;
  }
  else if(sc_BlendMode_ColoredGlass)
  {
    shadowAlpha = clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h) * color.a;
  }
  else if(sc_BlendMode_Add)
  {
    shadowAlpha = clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h);
  }
  else if(sc_BlendMode_AddWithAlphaFactor)
  {
    shadowAlpha = clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h) * color.a;
  }
  else if(sc_BlendMode_Screen)
  {
    shadowAlpha = dot(color.rgb, half3(0.33333)) * color.a;
  }
  else if(sc_BlendMode_Min)
  {
    shadowAlpha = (1.0 - clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h));// * color.a;
  }
  else if(sc_BlendMode_Max)
  {
    shadowAlpha = clamp(dot(color.rgb, half3(1.0)), 0.0h, 1.0h);// * color.a;
  }
  return shadowAlpha;
}

half4 evaluateShadowCasterColor(half3 shadowCasterColor, half alpha,constant FSShadowsUniforms& shadowsUniforms[[function_constant(sc_ProjectiveShadowsCaster)]]) {
  half shadowTerm = shadowsUniforms.sc_ShadowDensity * alpha;
  half3 shadowColor = mix(half3(shadowsUniforms.sc_ShadowColor.rgb), half3(shadowsUniforms.sc_ShadowColor.rgb) * shadowCasterColor, half(shadowsUniforms.sc_ShadowColor.a));
  return half4(shadowColor.r, shadowColor.g, shadowColor.b, shadowTerm);
}

/*
vec4 evaluateShadowCasterColor(float3 shadowCasterColor, float alpha) {
    float shadowTerm = sc_ShadowDensity * alpha;
    float3 shadowColor = mix(sc_ShadowColor.rgb, sc_ShadowColor.rgb * shadowCasterColor, sc_ShadowColor.a);
    return vec4(shadowColor.r, shadowColor.g, shadowColor.b, shadowTerm);
}

#ifdef FRAGMENT_SHADER
float getShadowTexClipFactor() {
    vec2 uv = abs(varShadowTex - 0.5);
    float clipFactor = max(uv.x, uv.y);
    return step(clipFactor, 0.5); //clipFactor > 0.5 = 0.0, clipFactor <= 0.5 = 1.0
}

//[DEPRECATED] Don't use this method in 77 and higher versions
float getShadowSample() {
    vec2 shadowSample = texture2D(sc_ShadowTexture, varShadowTex).ra;
    float result = shadowSample.x;
#if (sc_ExporterVersion >= 77) //10.36
    result = shadowSample.y; // for backward capability in Studio
#endif // sc_ExporterVersion 77
#if (sc_ExporterVersion >= 68)
    result = 1.0 - result * sc_ShadowDensity * getShadowTexClipFactor(); //r=0.0 - lit, r>0.0 - shade
#else
    result = clamp(result + 1.0 - getShadowTexClipFactor(), 0.0, 1.0); //r=1.0 - lit, r<1.0 - shade
#endif // sc_ExporterVersion 68
    return result;
}
//---------------------------------------------------------

*/

float2 getProjectedTexCoords(constant VSUniforms& uniforms,constant VSShadowsUniforms& shadowsUniforms[[function_constant(sc_ProjectiveShadowsReceiver)]], float4 vertexPosition) {
  float4 worldPos;
  if (sc_RenderingSpace == WorldSpace)
  {
    worldPos = vertexPosition;
  }
  else if (sc_RenderingSpace == ObjectSpace)
  {
    worldPos = uniforms.sc_ModelMatrix * vertexPosition;
  }
  else
  {
    worldPos = float4(0.0,0.0,0.0,0.0);
  }
  float4 projectedUVs = fixProjectionForMetal(shadowsUniforms.sc_ProjectorMatrix * worldPos);
  return projectedUVs.xy / projectedUVs.w * 0.5 + 0.5;
}


float getShadowTexClipFactor(float2 varShadowTex) {
  float2 uv = abs(varShadowTex - 0.5);
  float clipFactor = max(uv.x, uv.y);
  return step(clipFactor, 0.5); //clipFactor > 0.5 = 0.0, clipFactor <= 0.5 = 1.0
}

half3 evaluateShadow(texture2d<half> sc_ShadowTexture,
                      sampler sc_ShadowTextureSampler,constant FSShadowsUniforms& uniforms, half3 inputColor, float2 varShadowTex) {
  half4 shadowSample = sampleTextureSimple(sc_ShadowTexture,sc_ShadowTextureSampler, varShadowTex) * half(getShadowTexClipFactor(varShadowTex));
  half3 shadowColor = mix(half3(uniforms.sc_ShadowColor.rgb), half3(uniforms.sc_ShadowColor.rgb) * shadowSample.rgb, half(uniforms.sc_ShadowColor.a));
  half shadowDensity = shadowSample.a * half(uniforms.sc_ShadowDensity);
  return mix(half3(inputColor), half3(inputColor) * shadowColor, shadowDensity);
}

#endif
