//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "funconst.metal"
#else
#include <funconst.metal>
#endif

constant bool HASCOLORATTRIB [[function_constant(FUNCONST_VS_BIAS+0)]];
constant bool sc_VertexBlending [[function_constant(FUNCONST_VS_BIAS+1)]];
constant bool sc_VertexBlendingUseNormals[[function_constant(FUNCONST_VS_BIAS+2)]];
constant int sc_SkinBonesCount [[function_constant(FUNCONST_VS_BIAS+3)]];
constant bool SkinEnabled = sc_SkinBonesCount>0;
constant int MaxSkinBonesCount = 12;
constant int MaxShapesAttributes = 6;
constant int sc_RenderingSpace[[function_constant(FUNCONST_VS_BIAS+4)]];
constant bool sc_ProjectiveShadowsCaster[[function_constant(FUNCONST_VS_BIAS+5)]];
constant bool sc_ProjectiveShadowsReceiver[[function_constant(FUNCONST_VS_BIAS+6)]];
constant bool needShadowsUnfiforms = sc_ProjectiveShadowsCaster || sc_ProjectiveShadowsReceiver;
constant bool STD_ENABLE_TEXTURE1 [[function_constant(FUNCONST_VS_BIAS+7)]];
constant bool ENABLE_VERTEX_COLOR_BASE [[function_constant(FUNCONST_VS_BIAS+8)]];
constant bool RENDER_CONSTANT_COLOR [[function_constant(FUNCONST_VS_BIAS+9)]];
constant int sc_ExporterVersion [[function_constant(FUNCONST_VS_BIAS+10)]];
constant int SC_DEPTH_BUFFER_MODE_REGULAR [[function_constant(FUNCONST_GLOBAL_BIAS+11)]];
constant int SC_DEPTH_BUFFER_MODE_LOGARITHMIC [[function_constant(FUNCONST_GLOBAL_BIAS+12)]];
constant int sc_DepthBufferMode [[function_constant(FUNCONST_VS_BIAS+13)]];

struct Varyings {
    float4 position [[position]];
    float3 varNormal;
    float3 varTangent;
    half varBitangentSign;
    float3 varPos;
    float2 varTex0;
    float2 varTex1[[function_constant(STD_ENABLE_TEXTURE1)]];
    float4 varColor;
    float4 varScreenPos;
    float2 varScreenTexturePos;
    float2 varShadowTex[[function_constant(sc_ProjectiveShadowsReceiver)]];
};

struct sc_Vertex_t {
    float4 position;
    float3 normal;
    float3 tangent;
    float2 texture0;
    float2 texture1[[function_constant(STD_ENABLE_TEXTURE1)]];
    float4 color[[function_constant(HASCOLORATTRIB)]];
};

struct Vertex
{
    float3 position [[attribute(0)]];
    float3 normal [[attribute(1)]];
    float4 tangent [[attribute(2)]];
    float2 texture0 [[attribute(3)]];
    float2 texture1 [[attribute(4),function_constant(STD_ENABLE_TEXTURE1)]];
    float4 boneData[[attribute(5),function_constant(SkinEnabled)]];
    float4 color [[attribute(6),function_constant(HASCOLORATTRIB)]];
};

struct BlendShapeVertex
{
  packed_float3 p;
  packed_float3 n;
};

struct ShapesData
{
    BlendShapeVertex shape[MaxShapesAttributes];
};

struct VSUniformsSkin
{
    float4x4 sc_SkinBonesModelMatrices[MaxSkinBonesCount];
    float3x3 sc_SkinBonesNormalMatrices[MaxSkinBonesCount];
};

#define TEXTRANSFORMNAME(texName) texName##Transform

struct VSUniforms
{
    float4x4 sc_ViewProjectionMatrix;
    float4x4 sc_ModelViewMatrix;
    float4x4 sc_ModelViewProjectionMatrix;
    sc_Camera_t sc_Camera;
    float4x4 sc_ModelMatrix;
    float3x3 sc_NormalMatrix;
    float4 weights0;
    float4 weights1;
    float3x3 BASETEXNAMETRANSFORM;
    float3x3 MASKTEXNAMETRANSFORM;
    float4 sc_GeometryInfo; // vec4: x - primitive count, y - instance count, z - 0, w - 0
};

float getInstanceRatio(float instanceCount, ushort instanceId) {
    return (instanceCount > 1.0) ? float(instanceId) / (instanceCount - 1.0) : 0.0;
}

#if defined(CMDLINE)
#include "metalstd_shadows.metal"
#else
#include <metalstd_shadows.metal>
#endif

float3x3 to3x3(float4x4 tm)
               {
                   float3x3 result;
                   result[0].xyz = tm[0].xyz;
                   result[1].xyz = tm[1].xyz;
                   result[2].xyz = tm[2].xyz;
                   return result;
               }

void sc_SkinVertex(thread sc_Vertex_t& v,float4 boneData, VSUniformsSkin skinUniforms) {
    float4 weight = fract(boneData);
    weight.x = 1.0 - dot(weight.yzw, float3(1.0, 1.0, 1.0)); // because only x clamped to 0.99 (max weighted component)
    
    v.position.xyz = (skinUniforms.sc_SkinBonesModelMatrices[int(boneData[0])] * v.position * weight.x +
                      skinUniforms.sc_SkinBonesModelMatrices[int(boneData[1])] * v.position * weight.y +
                      skinUniforms.sc_SkinBonesModelMatrices[int(boneData[2])] * v.position * weight.z +
                      skinUniforms.sc_SkinBonesModelMatrices[int(boneData[3])] * v.position * weight.w).xyz;

    float3x3 normalMatrix0 = skinUniforms.sc_SkinBonesNormalMatrices[int(boneData[0])];
    float3x3 normalMatrix1 = skinUniforms.sc_SkinBonesNormalMatrices[int(boneData[1])];
    float3x3 normalMatrix2 = skinUniforms.sc_SkinBonesNormalMatrices[int(boneData[2])];
    float3x3 normalMatrix3 = skinUniforms.sc_SkinBonesNormalMatrices[int(boneData[3])];
    
    v.normal = normalMatrix0 * v.normal * weight.x +
    normalMatrix1 * v.normal * weight.y +
    normalMatrix2 * v.normal * weight.z +
    normalMatrix3 * v.normal * weight.w;
    v.tangent = normalMatrix0 * v.tangent * weight.x +
    normalMatrix1 * v.tangent * weight.y +
    normalMatrix2 * v.tangent * weight.z +
    normalMatrix3 * v.tangent * weight.w;
}

float4 applyDepthAlgorithm(float4 screenPosition, constant sc_Camera_t& sc_Camera) {
    if (sc_DepthBufferMode == SC_DEPTH_BUFFER_MODE_LOGARITHMIC) {
        float fCoefficient = 2.0 / log2(sc_Camera.clipPlanes.y + 1.0);
        screenPosition.z = (log2(max(sc_Camera.clipPlanes.x, 1.0 + screenPosition.w)) * fCoefficient - 1.0) * screenPosition.w;
    }
    return screenPosition;
}


float4 sc_ObjectToScreen(float4 objectPosition,constant VSUniforms& unforms) {
    float4 result;
    if (sc_RenderingSpace == ScreenSpace) {
        result = objectPosition;
    } else if (sc_RenderingSpace == WorldSpace) {
        result = unforms.sc_ViewProjectionMatrix * objectPosition;
    } else if (sc_RenderingSpace == ObjectSpace) {
        result = unforms.sc_ModelViewProjectionMatrix * objectPosition;
    } else {
        // sc_RenderingSpace == ScreenSpaceMV
        result = (unforms.sc_ModelViewMatrix * objectPosition) * float4(1.0 / unforms.sc_Camera.aspect, 1.0, 1.0, 1.0);
    }
    // applyDepthAlgorithm does not have sense for ScreenSpace and ScreenSpaceMV - but done to be compatible with OpenGL
    return fixProjectionForMetal(applyDepthAlgorithm(result,unforms.sc_Camera));
    
}

void blendTargetShape(thread sc_Vertex_t& v,
                      float3 position,
                     float weight) {
    v.position.xyz += position * weight;
}

void blendTargetShapeWithNormal(thread sc_Vertex_t& v,
                      float3 position,
                      float3 normal,
                      float weight) {
    v.position.xyz += position * weight;
    v.normal += normal * weight;
}

void sc_BlendVertex(thread sc_Vertex_t& v,constant VSUniforms& uniforms,
                    BlendShapeVertex shape0[[function_constant(sc_VertexBlending)]],
                    BlendShapeVertex shape1[[function_constant(sc_VertexBlending)]],
                    BlendShapeVertex shape2[[function_constant(sc_VertexBlending)]],
                    BlendShapeVertex shape3[[function_constant(sc_VertexBlending)]],
                    BlendShapeVertex shape4[[function_constant(sc_VertexBlending)]],
                    BlendShapeVertex shape5[[function_constant(sc_VertexBlending)]]) {
    if( sc_VertexBlendingUseNormals )
    {
      blendTargetShapeWithNormal(v, shape0.p, shape0.n, uniforms.weights0.x);
      blendTargetShapeWithNormal(v, shape1.p, shape1.n, uniforms.weights0.y);
      blendTargetShapeWithNormal(v, shape2.p, shape2.n, uniforms.weights0.z);
      blendTargetShapeWithNormal(v, shape3.p, shape3.n, uniforms.weights0.w);
      blendTargetShapeWithNormal(v, shape4.p, shape4.n, uniforms.weights1.x);
      blendTargetShapeWithNormal(v, shape5.p, shape5.n, uniforms.weights1.y);
    }
    else
    {
        blendTargetShape(v, shape0.p, uniforms.weights0.x);
        blendTargetShape(v, shape1.p, uniforms.weights0.y);
        blendTargetShape(v, shape2.p, uniforms.weights0.z);
        blendTargetShape(v, shape3.p, uniforms.weights0.w);
        blendTargetShape(v, shape4.p, uniforms.weights1.x);
        blendTargetShape(v, shape5.p, uniforms.weights1.y);
    }
}

sc_Vertex_t sc_LoadVertexAttributes(Vertex vbvertex) {
    sc_Vertex_t v;
    v.position = float4(vbvertex.position,1.0);
    v.normal = vbvertex.normal;
    v.tangent = vbvertex.tangent.xyz;
    v.texture0 = vbvertex.texture0;
    if( STD_ENABLE_TEXTURE1 )
    {
      v.texture1 = vbvertex.texture1;
    }
    else
    {
      v.texture1 = float2(1.0,1.0);
    }
    if(HASCOLORATTRIB)
    {
        v.color = vbvertex.color;
    }
    return v;
}

void sc_ProcessVertexVS(thread Varyings& varyings,
                      sc_Vertex_t v,
                      Vertex vtx,
                      constant VSUniforms& uniforms,
                      BlendShapeVertex shape0[[function_constant(sc_VertexBlending)]],
                      BlendShapeVertex shape1[[function_constant(sc_VertexBlending)]],
                      BlendShapeVertex shape2[[function_constant(sc_VertexBlending)]],
                      BlendShapeVertex shape3[[function_constant(sc_VertexBlending)]],
                      BlendShapeVertex shape4[[function_constant(sc_VertexBlending)]],
                      BlendShapeVertex shape5[[function_constant(sc_VertexBlending)]],
                      constant VSUniformsSkin& skinUniforms[[function_constant(SkinEnabled)]],
                      constant VSShadowsUniforms& shadowsUniforms[[function_constant(sc_ProjectiveShadowsReceiver)]]) {

    if( sc_VertexBlending )
    {
      sc_BlendVertex(v,uniforms,shape0,shape1,shape2,shape3,shape4,shape5);
    }

    if( SkinEnabled )
    {
        sc_SkinVertex(v,vtx.boneData,skinUniforms);
    }
  
    float4 screenPosition = sc_ObjectToScreen(v.position,uniforms);

    if (sc_RenderingSpace == ScreenSpace || sc_RenderingSpace == ScreenSpaceMV)
    {
        varyings.varPos = screenPosition.xyz;
        varyings.varNormal = v.normal;
        varyings.varTangent = v.tangent;
    }
    else if (sc_RenderingSpace == WorldSpace)
    {
        varyings.varPos = v.position.xyz;
        varyings.varNormal = v.normal;
        varyings.varTangent = v.tangent;
    }
    else if (sc_RenderingSpace == ObjectSpace)
    {
        varyings.varPos = (uniforms.sc_ModelMatrix * v.position).xyz;
        varyings.varNormal = uniforms.sc_NormalMatrix * v.normal;
        varyings.varTangent = uniforms.sc_NormalMatrix * v.tangent;
    }

    // Must make sure we normalize the tangent basis to avoid issues with scale!
    // If B = cross(N, T), then cross(2N, 2T) != 2B! It's actually =4B due to the definition of the cross product!
    // This way even uniform scale would skew the TBN matrix and mess up baked normal maps. So we must normalize.
    varyings.varNormal = normalize(varyings.varNormal);
    varyings.varTangent = normalize(varyings.varTangent);
    varyings.varBitangentSign = vtx.tangent.w;
    
    varyings.varTex0 = v.texture0;
    if( STD_ENABLE_TEXTURE1 )
    {
      varyings.varTex1 = v.texture1;
    }
    if( HASCOLORATTRIB )
    {
        varyings.varColor = v.color;
    }

    varyings.varScreenPos = screenPosition;
    varyings.varScreenTexturePos = screenPosition.xy / screenPosition.w * 0.5 + 0.5;
  
    if(sc_ProjectiveShadowsReceiver)
    {
        varyings.varShadowTex = getProjectedTexCoords(uniforms,shadowsUniforms,v.position);
      //varyings.varShadowTex.y = 1.0-varyings.varShadowTex.y;
    }

    varyings.position = screenPosition;
}


#define VSMAIN() vertex Varyings vs_main(Vertex vtx [[ stage_in ]], \
                  constant VSUniforms& uniforms [[ buffer(2) ]], \
                  constant BlendShapeVertex* shape0[[buffer(3),function_constant(sc_VertexBlending)]], \
                  constant BlendShapeVertex* shape1[[buffer(4),function_constant(sc_VertexBlending)]], \
                  constant BlendShapeVertex* shape2[[buffer(5),function_constant(sc_VertexBlending)]], \
                  constant BlendShapeVertex* shape3[[buffer(6),function_constant(sc_VertexBlending)]], \
                  constant BlendShapeVertex* shape4[[buffer(7),function_constant(sc_VertexBlending)]], \
                  constant BlendShapeVertex* shape5[[buffer(8),function_constant(sc_VertexBlending)]], \
                  uint vid [[vertex_id,function_constant(sc_VertexBlending)]], \
                  uint iid [[instance_id]], \
                  constant VSUniformsSkin& uniformsSkin[[ buffer(9),function_constant(SkinEnabled)]], \
                  constant VSShadowsUniforms& uniformsShadows[[buffer(10),function_constant(sc_ProjectiveShadowsReceiver)]] )

#define VSMAIN_WITH_RETTYPE(RETTYPE) vertex RETTYPE vs_main(Vertex vtx [[ stage_in ]], \
                    constant VSUniforms& uniforms [[ buffer(2) ]], \
                    constant BlendShapeVertex* shape0[[buffer(3),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape1[[buffer(4),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape2[[buffer(5),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape3[[buffer(6),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape4[[buffer(7),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape5[[buffer(8),function_constant(sc_VertexBlending)]], \
                    uint vid [[vertex_id,function_constant(sc_VertexBlending)]], \
                    uint iid [[instance_id]], \
                    constant VSUniformsSkin& uniformsSkin[[ buffer(9),function_constant(SkinEnabled)]], \
                    constant VSShadowsUniforms& uniformsShadows[[buffer(10),function_constant(sc_ProjectiveShadowsReceiver)]] )

#define VSMAIN_WITH_VERTEXTYPE_RETTYPE_AND_UNIFORMTYPE(VERTEXTYPE, RETTYPE, UNIFORMTYPE) vertex RETTYPE vs_main(VERTEXTYPE customVertex [[ stage_in ]], \
                    constant VSUniforms& uniforms [[ buffer(2) ]], \
                    constant BlendShapeVertex* shape0[[buffer(3),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape1[[buffer(4),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape2[[buffer(5),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape3[[buffer(6),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape4[[buffer(7),function_constant(sc_VertexBlending)]], \
                    constant BlendShapeVertex* shape5[[buffer(8),function_constant(sc_VertexBlending)]], \
                    uint vid [[vertex_id,function_constant(sc_VertexBlending)]], \
                    uint iid [[instance_id]], \
                    constant VSUniformsSkin& uniformsSkin[[ buffer(9),function_constant(SkinEnabled)]], \
                    constant VSShadowsUniforms& uniformsShadows[[buffer(10),function_constant(sc_ProjectiveShadowsReceiver)]], \
                    constant UNIFORMTYPE& customUniforms [[ buffer(11) ]])

#define sc_ProcessVertex(v) Varyings varyings; \
        sc_ProcessVertexVS(varyings,v,vtx,uniforms,shape0[vid],shape1[vid],shape2[vid],shape3[vid],shape4[vid],shape5[vid],uniformsSkin,uniformsShadows); \
        return varyings;

#define sc_ProcessVertexWithOrg(v,orgVertex) Varyings varyings; \
        sc_ProcessVertexVS(varyings,v,orgVertex,uniforms,shape0[vid],shape1[vid],shape2[vid],shape3[vid],shape4[vid],shape5[vid],uniformsSkin,uniformsShadows); \
        return varyings;

#define sc_ProcessVertexNoReturn(v) Varyings varyings; \
sc_ProcessVertexVS(varyings,v,vtx,uniforms,shape0[vid],shape1[vid],shape2[vid],shape3[vid],shape4[vid],shape5[vid],uniformsSkin,uniformsShadows); \

