#if defined(CMDLINE)
#include "metal_std.metal"
#else
#include <metal_std.metal>
#endif


struct Vertex
{
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct ColorInOut {
    float4 position [[position]];
    half2 texture0;
};

constant bool FORMAT_NV12 [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant bool FORMAT_NV21 [[function_constant(FUNCONST_VS_USER_BIAS+1)]];

// vertex shader function
vertex ColorInOut vs_main(Vertex vtx [[ stage_in ]])
{
    ColorInOut out;
    out.position = float4(vtx.position, 0.0, 1.0);
    out.texture0 = half2(vtx.texture0);
    out.texture0.y = 1.0f-out.texture0.y;
    return out;
}



half3 getRGB(half3 yuv) {
    const half3x3 yuv2rgb = half3x3(1.0, 1.0, 1.0,
                                      0.0, -0.337633, 1.732446,
                                      1.370705, -0.698001, 0.0);
    const half3 add       = half3(-0.6853525, 0.517817, -0.866223);
    return yuv2rgb * yuv + add;
}

// fragment shader function
fragment half4 fs_main(ColorInOut in [[stage_in]],
                       texture2d<half> yTexture,
                       sampler yTextureSampler,
                       texture2d<half> uvTexture,
                       sampler uvTextureSampler)
{
    half yVal = yTexture.sample(yTextureSampler, float2(in.texture0)).r;
    half2 uv  = uvTexture.sample(uvTextureSampler, float2(in.texture0)).rg;
    if (FORMAT_NV12) {
        return half4(getRGB(half3(yVal, uv[0], uv[1])), 1.0);
    } else if (FORMAT_NV21) {
        return half4(getRGB(half3(yVal, uv[1], uv[0])), 1.0);
    } else {
        assert(false);
        return half4(1.0,1.0,1.0,1.0);
    }
}
