//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_fs.metal"
#include "metalstd_vs.metal"
#else
#include <metal_std.metal>
#include <metalstd_fs.metal>
#include <metalstd_vs.metal>
#endif

constant int NUM_COLORS [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant int NUM_TEXTURES [[function_constant(FUNCONST_VS_USER_BIAS+1)]];
constant int NUM_STAGES [[function_constant(FUNCONST_VS_USER_BIAS+2)]];

constant int STAGE0_RGB_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+3)]];
constant int STAGE0_RGB_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+4)]];
constant int STAGE0_ALPHA_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+5)]];
constant int STAGE0_ALPHA_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+6)]];
constant int STAGE0_RGB_OP [[function_constant(FUNCONST_VS_USER_BIAS+7)]];
constant int STAGE0_ALPHA_OP [[function_constant(FUNCONST_VS_USER_BIAS+8)]];

constant int STAGE1_RGB_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+9)]];
constant int STAGE1_RGB_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+10)]];
constant int STAGE1_ALPHA_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+11)]];
constant int STAGE1_ALPHA_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+12)]];
constant int STAGE1_RGB_OP [[function_constant(FUNCONST_VS_USER_BIAS+13)]];
constant int STAGE1_ALPHA_OP [[function_constant(FUNCONST_VS_USER_BIAS+14)]];

constant int STAGE2_RGB_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+15)]];
constant int STAGE2_RGB_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+16)]];
constant int STAGE2_ALPHA_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+17)]];
constant int STAGE2_ALPHA_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+18)]];
constant int STAGE2_RGB_OP [[function_constant(FUNCONST_VS_USER_BIAS+19)]];
constant int STAGE2_ALPHA_OP [[function_constant(FUNCONST_VS_USER_BIAS+20)]];

constant int STAGE3_RGB_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+21)]];
constant int STAGE3_RGB_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+22)]];
constant int STAGE3_ALPHA_ARG1 [[function_constant(FUNCONST_VS_USER_BIAS+23)]];
constant int STAGE3_ALPHA_ARG2 [[function_constant(FUNCONST_VS_USER_BIAS+24)]];
constant int STAGE3_RGB_OP [[function_constant(FUNCONST_VS_USER_BIAS+25)]];
constant int STAGE3_ALPHA_OP [[function_constant(FUNCONST_VS_USER_BIAS+26)]];

constant bool ALPHATEST [[function_constant(FUNCONST_VS_USER_BIAS+27)]];

constant bool hasColor = NUM_COLORS>0;
constant bool hasTex0 = NUM_TEXTURES>0;
constant bool hasTex1 = NUM_TEXTURES>1;
constant bool hasTex2 = NUM_TEXTURES>2;
constant bool hasTex3 = NUM_TEXTURES>3;

struct MPVaryings
{
    float4 position [[position]];
    float4 colorVarying[[function_constant(hasColor)]];
    float2 texture0[[function_constant(hasTex0)]];
    float2 texture1[[function_constant(hasTex1)]];
    float2 texture2[[function_constant(hasTex2)]];
    float2 texture3[[function_constant(hasTex3)]];
};

struct MPUniforms
{
    float4x4 sc_ViewProjectionMatrix;
    sc_Camera_t sc_Camera;
};

struct MPVertex
{
    float3 position [[attribute(0)]];
    float4 color [[attribute(1)]];
    float2 inputTextureCoordinate0 [[attribute(2)]];
    float2 inputTextureCoordinate1 [[attribute(3)]];
    float2 inputTextureCoordinate2 [[attribute(4)]];
    float2 inputTextureCoordinate3 [[attribute(5)]];
};

vertex MPVaryings vs_main(MPVertex vtx [[ stage_in ]],
                     constant MPUniforms& uniforms [[ buffer(2) ]]) {
    
    MPVaryings out;
    out.position = uniforms.sc_ViewProjectionMatrix * float4(vtx.position, 1.0);
    out.position = applyDepthAlgorithm(out.position,uniforms.sc_Camera);
    out.position = fixProjectionForMetal(out.position);
    
    if (NUM_COLORS>0)
    {
        out.colorVarying = vtx.color;
    }
    
    if( NUM_TEXTURES>0 )
    {
        out.texture0 = float2(vtx.inputTextureCoordinate0.x,1.0-vtx.inputTextureCoordinate0.y);
    }
    
    if( NUM_TEXTURES>1 )
    {
        out.texture1 = float2(vtx.inputTextureCoordinate1.x,1.0-vtx.inputTextureCoordinate1.y);
    }
    
    if( NUM_TEXTURES>2 )
    {
        out.texture2 = float2(vtx.inputTextureCoordinate2.x,1.0-vtx.inputTextureCoordinate2.y);
    }
    
    if (NUM_TEXTURES>3)
    {
        out.texture3 = float2(vtx.inputTextureCoordinate3.x,1.0-vtx.inputTextureCoordinate3.y);
    }

    return out;
}

#define TEXARG_CURRENT 0
#define TEXARG_DIFFUSE 1
#define TEXARG_TEXTURE 2

#define TEXOP_ARGUMENT1    0
#define TEXOP_ADD            1
#define TEXOP_SUBTRACT    2
#define TEXOP_MODULATE    3
#define TEXOP_MODULATE2X    4
#define TEXOP_MODULATE4X    5

half4 stage(int RGB_ARG1,int RGB_ARG2,int ALPHA_ARG1,int ALPHA_ARG2,int RGB_OP,int ALPHA_OP, texture2d<half> texture,
            sampler textureSampler,float2 textureCoordinate,half4 color,half4 colorVarying)
{
    half4 arg1;
    half4 arg2;
    half4 colorTex;
    if ((RGB_ARG1==TEXARG_TEXTURE) || (ALPHA_ARG1==TEXARG_TEXTURE) || (RGB_ARG2==TEXARG_TEXTURE) || (ALPHA_ARG2==TEXARG_TEXTURE))
    {
        colorTex=sampleTextureSimple(texture,textureSampler,textureCoordinate);
    }
    // load arg1
    if(RGB_ARG1==TEXARG_CURRENT)
    {
        arg1.xyz = color.xyz;
    }
    else if(RGB_ARG1==TEXARG_DIFFUSE)
    {
        arg1.xyz = colorVarying.xyz;
    }
    else if(RGB_ARG1==TEXARG_TEXTURE)
    {
        arg1.xyz = colorTex.xyz;
    }
    if(ALPHA_ARG1==TEXARG_CURRENT)
    {
        arg1.w = color.w;
    }
    else if(ALPHA_ARG1==TEXARG_DIFFUSE)
    {
        arg1.w = colorVarying.w;
    }
    else if(ALPHA_ARG1==TEXARG_TEXTURE)
    {
        arg1.w = colorTex.w;
    }
    // load arg2
    if(RGB_ARG2==TEXARG_CURRENT)
    {
        arg2.xyz = color.xyz;
    }
    else if(RGB_ARG2==TEXARG_DIFFUSE)
    {
        arg2.xyz = colorVarying.xyz;
    }
    else if(RGB_ARG2==TEXARG_TEXTURE)
    {
        arg2.xyz = colorTex.xyz;
    }
    if(ALPHA_ARG2==TEXARG_CURRENT)
    {
        arg2.w = color.w;
    }
    else if(ALPHA_ARG2==TEXARG_DIFFUSE)
    {
        arg2.w = colorVarying.w;
    }
    else if(ALPHA_ARG2==TEXARG_TEXTURE)
    {
        arg2.w = colorTex.w;
    }
    // color op
    if(RGB_OP==TEXOP_ARGUMENT1)
    {
        color.xyz = arg1.xyz;
    }
    else if(RGB_OP==TEXOP_ADD)
    {
        color.xyz = arg1.xyz+arg2.xyz;
    }
    else if(RGB_OP==TEXOP_SUBTRACT)
    {
        color.xyz = arg1.xyz-arg2.xyz;
    }
    else if(RGB_OP==TEXOP_MODULATE)
    {
        color.xyz = arg1.xyz*arg2.xyz;
    }
    else if(RGB_OP==TEXOP_MODULATE2X)
    {
        color.xyz = arg1.xyz*arg2.xyz;
        color.xyz = color.xyz+color.xyz;
    }
    else // modulate 4x
    {
        color.xyz=arg1.xyz*arg2.xyz;
        color.xyz=color.xyz*4.0;
    }
// alpha op
    if(ALPHA_OP==TEXOP_ARGUMENT1)
    {
        color.w = arg1.w;
    }
    else if(ALPHA_OP==TEXOP_ADD)
    {
        color.w = arg1.w+arg2.w;
    }
    else if(ALPHA_OP==TEXOP_SUBTRACT)
    {
        color.w = arg1.w-arg2.w;
    }
    else if(ALPHA_OP==TEXOP_MODULATE)
    {
        color.w = arg1.w*arg2.w;
    }
    else if(ALPHA_OP==TEXOP_MODULATE2X)
    {
        color.w = arg1.w*arg2.w;
        color.w = color.w+color.w;
    }
    else // modulate 4x
    {
        color.w=arg1.w*arg2.w;
        color.w=color.w*4.0;
    }
    return color;
}

fragment half4 fs_main(MPVaryings in [[stage_in]],
                       texture2d<half> texture0[[function_constant(hasTex0)]],
                       sampler texture0Sampler[[function_constant(hasTex0)]],
                       texture2d<half> texture1[[function_constant(hasTex1)]],
                       sampler texture1Sampler[[function_constant(hasTex1)]],
                       texture2d<half> texture2[[function_constant(hasTex2)]],
                       sampler texture2Sampler[[function_constant(hasTex2)]],
                       texture2d<half> texture3[[function_constant(hasTex3)]],
                       sampler texture3Sampler[[function_constant(hasTex3)]] )
{
    
    half4 color;
    half4 colorVarying;
    if( hasColor )
    {
        colorVarying = half4(in.colorVarying);
    }
    
    if(NUM_STAGES==0)
    {
        color = colorVarying;
    }
    
    if(NUM_STAGES>0)
    {
        color = stage(STAGE0_RGB_ARG1,STAGE0_RGB_ARG2,STAGE0_ALPHA_ARG1,STAGE0_ALPHA_ARG2,STAGE0_RGB_OP,STAGE0_ALPHA_OP,texture0,texture0Sampler,in.texture0,color,colorVarying);
    }
    if(NUM_STAGES>1)
    {
        color = stage(STAGE1_RGB_ARG1,STAGE1_RGB_ARG2,STAGE1_ALPHA_ARG1,STAGE1_ALPHA_ARG2,STAGE1_RGB_OP,STAGE1_ALPHA_OP,texture1,texture1Sampler,in.texture1,color,colorVarying);
    }
    if(NUM_STAGES>2)
    {
        color = stage(STAGE2_RGB_ARG1,STAGE2_RGB_ARG2,STAGE2_ALPHA_ARG1,STAGE2_ALPHA_ARG2,STAGE2_RGB_OP,STAGE2_ALPHA_OP,texture2,texture2Sampler,in.texture2,color,colorVarying);
    }
    if(NUM_STAGES>3)
    {
        color = stage(STAGE3_RGB_ARG1,STAGE3_RGB_ARG2,STAGE3_ALPHA_ARG1,STAGE3_ALPHA_ARG2,STAGE3_RGB_OP,STAGE3_ALPHA_OP,texture3,texture3Sampler,in.texture3,color,colorVarying);
    }

    if (ALPHATEST)
    {
        if(color.w==0.0)discard_fragment();
    }
    return color;
}
