//-----------------------------------------------------------------------
// Copyright (c) 2018 Snap Inc.
//-----------------------------------------------------------------------

// rectangleTo2D.glsl is used directly for video output on mac to copy and convert from pixel buffer backed texture. For metal we need to copy still, and additionally, video frames have to be loaded as ARGB for full compatability and the metal texture must be BGRA so we need to convert here.

using namespace metal;

struct Vertex
{
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct ColorInOut {
    float4 position [[position]];
    half2 texture0;
};


// vertex shader function
vertex ColorInOut vs_main(Vertex vtx [[ stage_in ]])
{
    ColorInOut out;
    out.position = float4(vtx.position, 0.0, 1.0);
    out.texture0 = half2(vtx.texture0);
    out.texture0.y = 1.0f-out.texture0.y;
    return out;
}

// fragment shader function
fragment half4 fs_main(ColorInOut in [[stage_in]],
                                 texture2d<half> rectTexture,
                                sampler rectTextureSampler)
{
    //ARGB->BGRA
    return rectTexture.sample(rectTextureSampler, float2(in.texture0)).grab;
};

