//-----------------------------------------------------------------------
// Copyright (c) 2017 Snap Inc.
//-----------------------------------------------------------------------

#define SC_USE_USER_DEFINED_VS_MAIN

#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_vs.metal"
#include "metalstd_fs.metal"
#else
#include <metal_std.metal>
#include <metalstd_vs.metal>
#include <metalstd_fs.metal>
#endif

typedef float2 vec2;
typedef float3 vec3;
typedef float4 vec4;
typedef float4x4 mat4;

#define sc_TimeElapsed (uniforms.sc_Time.x)
#define sc_TimeDelta   (uniforms.sc_Time.y)

constant bool SOFT_SKIN [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant bool EYE_SHARPEN [[function_constant(FUNCONST_VS_USER_BIAS+1)]];
constant bool TEETH_WHITENING [[function_constant(FUNCONST_VS_USER_BIAS+2)]];
constant bool EYE_WHITENING [[function_constant(FUNCONST_VS_USER_BIAS+3)]];
constant bool ADD_NOISE [[function_constant(FUNCONST_VS_USER_BIAS+4)]];

constant bool NeedScreenTexture = SOFT_SKIN || EYE_SHARPEN || sc_FramebufferFetch==false;
constant bool NeedLookupTexture = EYE_WHITENING || TEETH_WHITENING;

struct FSUniforms
{
    float softSkinIntensity;
    float teethWhiteningIntensity;
    float eyeWhiteningIntensity;
    float sharpenEyeIntensity;
    float4 sc_Time;
};

constant float SOFTSKIN_RADIUS = 0.1;
constant float RETOUCH_EPSILON  = 0.000001;

struct VaryingsRetouch {
    float4 position [[position]];
    float3 varNormal;
    float3 varTangent;
    half varBitangentSign;
    float3 varPos;
    float2 varTex0;
    float2 varTex1[[function_constant(STD_ENABLE_TEXTURE1)]];
    float4 varColor;
    float4 varScreenPos;
    float2 varScreenTexturePos;
    float2 varShadowTex[[function_constant(sc_ProjectiveShadowsReceiver)]];

    float2 texCoord;
    float4 texCoord0;
    float4 texCoord1;
    float4 texCoord2;
    float4 texCoord3;
};


VSMAIN_WITH_RETTYPE(VaryingsRetouch)
{
    sc_Vertex_t v = sc_LoadVertexAttributes(vtx);

    
    VaryingsRetouch varRetouch;

    
    vec2 delta = vec2(uniforms.sc_Camera.aspect / 1280.0, 1.0 / 1280.0);
    
    varRetouch.texCoord = v.position.xy / v.position.w * 0.5 + vec2(0.5, 0.5);
    
    
    if(SOFT_SKIN)
    {
        //Soft skin coords
        varRetouch.texCoord0.xy = varRetouch.texCoord + vec2(-0.00694444, -0.00390625);
        varRetouch.texCoord1.xy = varRetouch.texCoord + vec2(-0.00694444, 0.00546875);
        varRetouch.texCoord2.xy = varRetouch.texCoord + vec2(0.00972222, -0.00390625);
        varRetouch.texCoord3.xy = varRetouch.texCoord + vec2(0.00972222, 0.00546875);
    }
    if(EYE_SHARPEN)
    {
        //Sharpening coords
        varRetouch.texCoord0.zw = varRetouch.texCoord + vec2(-delta.x, -delta.y);
        varRetouch.texCoord1.zw = varRetouch.texCoord + vec2(delta.x, -delta.y);
        varRetouch.texCoord2.zw = varRetouch.texCoord + vec2(-delta.x, delta.y);
        varRetouch.texCoord3.zw = varRetouch.texCoord + vec2(delta.x, delta.y);
    }


    sc_ProcessVertexNoReturn(v);
    
    varRetouch.position = varyings.position;
    varRetouch.varNormal = varyings.varNormal;
    varRetouch.varTangent = varyings.varTangent;
    varRetouch.varBitangentSign = varyings.varBitangentSign;
    varRetouch.varPos = varyings.varPos;
    varRetouch.varTex0 = varyings.varTex0;
    if( STD_ENABLE_TEXTURE1 )
    {
        varRetouch.varTex1 = varyings.varTex1;
    }
    varRetouch.varColor = varyings.varColor;
    varRetouch.varScreenPos = varyings.varScreenPos;
    varRetouch.varScreenTexturePos = varyings.varScreenTexturePos;
    if( sc_ProjectiveShadowsReceiver )
    {
        varRetouch.varShadowTex = varyings.varShadowTex;
    }
    return varRetouch;
}


vec4 mapColor(vec4 orgColor,DECLARETEXTUREOPTIONAL(lookupTexture,lookupTextureSampler,TEETH_WHITENING))
{
    const float pxSize = 512.0;
    
    float bValue = (orgColor.b * 255.0) / 4.0;
    
    vec2 mulB = clamp(floor(bValue) + vec2(0.0, 1.0), 0.0, 63.0);
    vec2 row = floor(mulB / 8.0 + RETOUCH_EPSILON);
    vec4 row_col = vec4(row, mulB - row * 8.0);
    vec4 lookup = orgColor.ggrr * (63.0/pxSize) + row_col * (64.0/pxSize) + (0.5/pxSize);
    float b1w = bValue - mulB.x;
    
    vec3 sampled1 = float3(sampleTextureSimple(lookupTexture,lookupTextureSampler, vec2(lookup.z, 1.0 - lookup.x)).rgb);
    vec3 sampled2 = float3(sampleTextureSimple(lookupTexture,lookupTextureSampler,vec2(lookup.w, 1.0 - lookup.y)).rgb);
    
    vec3 res = mix(sampled1, sampled2, b1w);
    return vec4(res, orgColor.a);
}

vec4 whitening(vec4 originalColor, float factor,DECLARETEXTUREOPTIONAL(lookupTexture,lookupTextureSampler,TEETH_WHITENING)) {
    vec4 color = mapColor(originalColor,lookupTexture,lookupTextureSampler);
    return mix(originalColor, color, factor);
}

 
half4 sharpen(half4 originalColor, float factor,DECLARETEXTUREOPTIONAL(sc_ScreenTexture,sc_ScreenTextureSampler,NeedScreenTexture),VaryingsRetouch varyings) {
    half4 accum = 5.0 * originalColor;
    accum -= sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord0.zw);
    accum -= sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord1.zw);
    accum -= sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord2.zw);
    accum -= sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord3.zw);
    half4 result = mix(originalColor, accum, (half)factor);
    return clamp(result, half(0.0), half(1.0));
}


vec4 getLuminance4(mat4 color) {
    const vec4 rgb2y = vec4(0.299, 0.587, 0.114, 0.0);
    return rgb2y * color;
}

float getLuminance(vec4 color) {
    const vec4 rgb2y = vec4(0.299, 0.587, 0.114, 0.0);
    return dot(color, rgb2y);
}

float rand(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898,78.233))) * 43758.5453);
}

float noise(vec2 co,constant FSUniforms& uniforms){
    float t=fract(sc_TimeElapsed);
    return (rand(co+0.07*t)+rand(co+0.11*t))/2.0;
}

vec4 getWeight(float intens, vec4 nextIntens) {
    // HACK 1/4/2019: Add a small float value to avoid infinity results and their propagation.
    vec4 lg = log(nextIntens / (intens + RETOUCH_EPSILON));
    lg *= lg;
    return exp(lg * (-1.0 / (2.0 * SOFTSKIN_RADIUS * SOFTSKIN_RADIUS)));
}

vec4 softSkin(vec4 originalColor, float factor,DECLARETEXTUREOPTIONAL(sc_ScreenTexture,sc_ScreenTextureSampler,NeedScreenTexture),VaryingsRetouch varyings,constant FSUniforms& uniforms) {
    vec4 screenColor = originalColor;
    float intens = getLuminance(screenColor);
    float sum = 1.0;
    
    mat4 nextColor;
    nextColor[0] = (float4)sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord0.xy);
    nextColor[1] = (float4)sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord1.xy);
    nextColor[2] = (float4)sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord2.xy);
    nextColor[3] = (float4)sampleTextureSimple(sc_ScreenTexture,sc_ScreenTextureSampler, varyings.texCoord3.xy);
    vec4 nextIntens = getLuminance4(nextColor);
    vec4 curr = 0.36787944 * getWeight(intens, nextIntens);
    sum += dot(curr, vec4(1.0));
    screenColor += nextColor * curr;

    if (ADD_NOISE)
    {
        float noiseVal=(noise(varyings.texCoord,uniforms) -0.5)/22.0;
        screenColor=screenColor/sum+float4(noiseVal,noiseVal,noiseVal,1.0);
    }
    else
    {
        screenColor=screenColor/sum;
    }

    screenColor = mix(originalColor, screenColor, factor);
    return screenColor;
}

// fragment shader function
fragment half4 fs_main(VaryingsRetouch in [[stage_in]],
                       constant FSUniforms& uniforms [[ buffer(2) ]],
                       DECLARETEXTUREOPTIONAL(lookupTexture,lookupTextureSampler,NeedLookupTexture),
                       DECLARETEXTURE(maskTexture,maskTextureSampler),
                       DECLARETEXTUREOPTIONAL(sc_ScreenTexture,sc_ScreenTextureSampler,NeedScreenTexture)
                       OPTIONAL_FRAMEBUFFERFETCH_PARAM())
{
    OPTIONAL_FRAMEBUFFERFETCH_VAR();
    
    half4 originalColor = getFramebufferColor((half4)frameBufferColor,TEXTUREASPARAM(sc_ScreenTexture),in.varScreenPos);
    half4 maskColor = sampleTextureSimple(maskTexture,maskTextureSampler, in.varTex1);
    half4 res = originalColor;
    if(SOFT_SKIN)
    {
        res = (half4)softSkin((float4)res, maskColor.r * uniforms.softSkinIntensity,sc_ScreenTexture,sc_ScreenTextureSampler,in,uniforms); // try softskin
    }
    if(EYE_SHARPEN)
    {
        res = sharpen(res, maskColor.b * uniforms.sharpenEyeIntensity,sc_ScreenTexture,sc_ScreenTextureSampler,in); // try softskin
    }
        float whiteningAlpha = 0.0;
    if(EYE_WHITENING)
    {
        whiteningAlpha += maskColor.b * uniforms.eyeWhiteningIntensity;
    }
    if(TEETH_WHITENING)
    {
        whiteningAlpha += maskColor.g * uniforms.teethWhiteningIntensity;
    }
    if (EYE_WHITENING || TEETH_WHITENING)
    {
        res = (half4)whitening((float4)res, whiteningAlpha,lookupTexture,lookupTextureSampler);// try white teeth and eyes
    }
    return res;
}
