//-----------------------------------------------------------------------
// Copyright (c) 2018 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "metal_std.metal"
#else
#include <metal_std.metal>
#endif


struct Vertex
{
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct ColorInOut {
    float4 position [[position]];
    half2 texture0;
};

struct Uniforms
{
    float3x3 screenTextureTransform;
    float screenTextureLOD;
};

constant bool SWAP_R_B_CHANNELS [[function_constant(FUNCONST_VS_USER_BIAS+0)]];

// vertex shader function
vertex ColorInOut vs_main(Vertex vtx [[ stage_in ]],
                          constant Uniforms& uniforms [[ buffer(2) ]])
{
    ColorInOut out;
    out.position = float4(vtx.position, 0.0, 1.0);
    out.texture0 = half2((uniforms.screenTextureTransform * float3(vtx.texture0, 1.0)).xy);
    out.texture0.y = 1.0f-out.texture0.y;
    return out;
}

// fragment shader function
fragment half4 fs_main(ColorInOut in [[stage_in]],
                       constant Uniforms& uniforms [[ buffer(2) ]],
                       texture2d<half> screenTexture,
                       sampler screenTextureSampler)
{
    if (SWAP_R_B_CHANNELS) {
        return screenTexture.sample(screenTextureSampler, float2(in.texture0), level(uniforms.screenTextureLOD)).bgra;
    } else {
        return screenTexture.sample(screenTextureSampler, float2(in.texture0), level(uniforms.screenTextureLOD));
    }
};

