//-----------------------------------------------------------------------
// Copyright (c) 2018 Snap Inc.
//-----------------------------------------------------------------------
#if defined(CMDLINE)
#include "metal_std.metal"
#include "metalstd_vs.metal"
#include "metalstd_fs.metal"
#else
#include <metal_std.metal>
#include <metalstd_vs.metal>
#include <metalstd_fs.metal>
#endif

struct TAVertex
{
    float2 position [[attribute(0)]];
    float2 texture0 [[attribute(1)]];
};

struct ColorInOut {
    float4 position [[position]];
    half2 texture0;
};

struct Uniforms
{
    float4 baseColor;
};

constant bool R [[function_constant(FUNCONST_VS_USER_BIAS+0)]];
constant bool RG [[function_constant(FUNCONST_VS_USER_BIAS+1)]];
constant bool RGBA [[function_constant(FUNCONST_VS_USER_BIAS+2)]];
constant bool COLOR_SOURCE [[function_constant(FUNCONST_VS_USER_BIAS+3)]];

// vertex shader function
vertex ColorInOut vs_main(TAVertex vtx [[ stage_in ]])
{
    ColorInOut out;
    out.position = float4(vtx.position, 0.0, 1.0);
    out.texture0 = half2(vtx.texture0.xy);
    return out;
}

// fragment shader function
fragment half4 fs_main(ColorInOut in [[stage_in]],
                       constant Uniforms& uniforms [[ buffer(2) ]],
                       texture2d<half> inputTexture,
                       sampler inputTextureSampler)
{
    half4 color;
    
    if (COLOR_SOURCE) {
        color = inputTexture.sample(inputTextureSampler, float2(in.texture0)) * half4(uniforms.baseColor);
    } else {
        color = half4(half3(uniforms.baseColor.rgb), inputTexture.sample(inputTextureSampler, float2(in.texture0)).r * half(uniforms.baseColor.a));
    }
    if (R) {
        return half4(color.a, 0.0, 0.0, color.a);
    }
    if (RG) {
        return half4(color.r, color.a, 0.0, color.a);
    }
    return color;
};

