(function() {
    window.serengeti = window.serengeti || {};

    window.serengeti.version = 1;

    window.serengeti.__updateContext__ = function(newContext) {
      let contextChanged = false;

      Object.keys(newContext).map(function(objectKey, index) {
        const value = newContext[objectKey];
        if (window.serengeti.context[objectKey] !== value) {
          window.serengeti.context[objectKey] = value;
          contextChanged = true;
        }
      });

      if (contextChanged && window.serengeti.onContextChanged) {
        window.serengeti.onContextChanged();
      }
    };

    window.serengeti.__keyboardWillAnimate__ = function(delta, duration) {
      if (window.serengeti.onKeyboardWillAnimate) {
        window.serengeti.onKeyboardWillAnimate(delta, duration);
      }
    };

    window.serengeti.dismiss = function() {
      window.webkit.messageHandlers.serengeti.postMessage({
        functionName: '__dismiss__'
      });
    };

    // Stories Promise-like objects to handle callbacks from
    // the embedded web view
    // guid -> {resolve: resolve, reject: reject}
    window.serengeti.callbacks = {};

    // More like a random string in UUID form...
    window.serengeti.guid = function() {
      function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
          .toString(16)
          .substring(1);
      }
      return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
        s4() + '-' + s4() + s4() + s4();
    };

    // All Serengeti-provided methods should be called using this function
    window.serengeti.invoke = function(functionName, argsDict) {
      return new Promise(function(resolve, reject) {
        var callbackId = window.serengeti.guid();

        // Stash away our callbacks for later
        window.serengeti.callbacks[callbackId] = {
          resolve: resolve,
          reject: reject
        };

        // And then send a message to the embedded web view
        window.webkit.messageHandlers.serengeti.postMessage({
          callbackId: callbackId,
          functionName: functionName,
          args: argsDict
        });
      });
    };

    // Called by the embedded webview to properly call the Promise callbacks
    window.serengeti.invokeCallback = function(callbackId, result) {
      var callback = window.serengeti.callbacks[callbackId];
      delete window.serengeti.callbacks[callbackId];

      if (result.error) {
        callback.reject(result.error);
      } else {
        callback.resolve(result.response);
      }
    };

    // Wrapper functions for Serengeti methods
    window.serengeti.alert = function(title, message) {
      return window.serengeti.invoke('alert', {
        title: title,
        message: message
      });
    };

    window.serengeti.openUrl = function(url, closeCurrent, additionalInfo) {
      return window.serengeti.invoke('openUrl', {
        url: url,
        closeCurrent: closeCurrent,
        additionalInfo: additionalInfo || null,
      });
    };

    window.serengeti.openSerengetiLink = function(link, shouldPush) {
      return window.serengeti.invoke('openSerengetiLink', {
        link: link,
        shouldPush: shouldPush,
      });
    };

    // style can be 'black', 'white', 'hidden'
    window.serengeti.setStatusBarStyle = function(style, animated) {
      return window.serengeti.invoke('setStatusBarStyle', {
        style: style,
        animated: animated
      });
    };

    // request is an object that can contain the following properties:
    // fsnPath, url, authenticated, headers, body, method
    // if fsnPath is set, the request will go to FSN using the given path,
    // otherwise the full URL will be used.
    window.serengeti.makeRequest = function(request) {
      return window.serengeti.invoke('makeRequest', request);
    };

    window.serengeti.log = function(severity, args) {
      message = '[JS ' + severity + '] ' + Array.from(args).join(' ');
      return window.serengeti.invoke('log', {
        message: message,
      });
    };

    window.serengeti.viewFinishedLoading = function(parameters) {
      return window.serengeti.invoke('viewFinishedLoading', parameters);
    };

    // Observables

    window.serengeti.observers = {};
    window.serengeti.addObserver = function(key, parameters, callback) {
      const id = window.serengeti.guid();

      const observer = {};
      observer.cancel = function() {
        delete window.serengeti.observers[id];
        window.serengeti.invoke('removeObserver', {observerId: id});
      };

      window.serengeti.observers[id] = callback;
      window.serengeti.invoke('addObserver', {observable: key, parameters: parameters, observerId: id});

      return observer;
    };

    window.serengeti.notifyObserver = function(observerId, parameters) {
      const observer = window.serengeti.observers[observerId];
      if (observer) {
        observer(parameters);
      }
    };

    // replace console.log with serengeti log for ease-of-use
    console.log = function() {
      window.serengeti.log('log', arguments);
    };
    console.info = function() {
      window.serengeti.log('info', arguments);
    };
    console.warn = function() {
      window.serengeti.log('warn', arguments);
    };
    console.error = function() {
      window.serengeti.log('error', arguments);
    };

    document.documentElement.style.webkitUserSelect='none';
    document.documentElement.style.webkitTouchCallout='none';
})();
